@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22073 $
+ * $Revision: 22817 $
  * $Author: rjongbloed $
- * $Date: 2009-02-16 11:21:01 +0000 (Mon, 16 Feb 2009) $
+ * $Date: 2009-06-05 02:37:23 +0000 (Fri, 05 Jun 2009) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -270,31 +270,15 @@ class OpalPluginStreamedAudioTranscoder : public OpalStreamedTranscoder, public
 {
   PCLASSINFO(OpalPluginStreamedAudioTranscoder, OpalStreamedTranscoder);
   public:
-    OpalPluginStreamedAudioTranscoder(const PluginCodec_Definition * codec, bool isEncoder, unsigned inputBits, unsigned outputBits);
+    OpalPluginStreamedAudioTranscoder(const PluginCodec_Definition * codec, bool isEncoder);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
     virtual bool AcceptComfortNoise() const { return comfortNoise; }
+    virtual int ConvertOne(int from) const;
   protected:
     bool comfortNoise;
 };
 
 
-class OpalPluginStreamedAudioEncoder : public OpalPluginStreamedAudioTranscoder
-{
-  PCLASSINFO(OpalPluginStreamedAudioEncoder, OpalPluginStreamedAudioTranscoder);
-  public:
-    OpalPluginStreamedAudioEncoder(const PluginCodec_Definition * codec, bool);
-    int ConvertOne(int _sample) const;
-};
-
-class OpalPluginStreamedAudioDecoder : public OpalPluginStreamedAudioTranscoder
-{
-  PCLASSINFO(OpalPluginStreamedAudioDecoder, OpalPluginStreamedAudioTranscoder);
-  public:
-    OpalPluginStreamedAudioDecoder(const PluginCodec_Definition * codec, bool);
-    int ConvertOne(int codedSample) const;
-};
-
-
 ///////////////////////////////////////////////////////////////////////////////
 
 #if OPAL_VIDEO
