@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21389 $
- * $Author: hfriederich $
- * $Date: 2008-10-20 21:14:44 +0000 (Mon, 20 Oct 2008) $
+ * $Revision: 21956 $
+ * $Author: rjongbloed $
+ * $Date: 2009-01-28 03:09:29 +0000 (Wed, 28 Jan 2009) $
  */
 
 #ifndef OPAL_H323_H323CAPS_H
@@ -1614,237 +1614,6 @@ class H323_G711Capability : public H323AudioCapability
     Speed    speed;
 };
 
-#if 0
-
-/**This class describes the G.728 codec capability.
- */
-class H323_G728Capability : public H323AudioCapability
-{
-  PCLASSINFO(H323_G728Capability, H323AudioCapability)
-
-  public:
-  /**@name Construction */
-  //@{
-    /**Create a new G.728 capability.
-     */
-    H323_G728Capability();
-  //@}
-
-  /**@name Overrides from class PObject */
-  //@{
-    /**Create a copy of the object.
-      */
-    virtual PObject * Clone() const;
-  //@}
-
-  /**@name Identification functions */
-  //@{
-    /**Get the sub-type of the capability. This is a code dependent on the
-       main type of the capability.
-     */
-    virtual unsigned GetSubType() const;
-
-    /**Get the name of the media data format this class represents.
-     */
-    virtual PString GetFormatName() const;
-  //@}
-};
-
-
-/**This class describes the G.729 codec capability.
- */
-class H323_G729Capability : public H323AudioCapability
-{
-  PCLASSINFO(H323_G729Capability, H323AudioCapability)
-
-  public:
-    /// Specific G.729 encoding algorithm.
-    enum Mode {
-      e_Normal,
-      e_AnnexA,
-      e_AnnexB,
-      e_AnnexA_AnnexB
-    };
-
-  /**@name Construction */
-  //@{
-    /**Create a new G.729 capability.
-     */
-    H323_G729Capability(
-      Mode mode 
-    );
-  //@}
-
-  /**@name Overrides from class PObject */
-  //@{
-    /**Create a copy of the object.
-      */
-    virtual PObject * Clone() const;
-  //@}
-
-  /**@name Identification functions */
-  //@{
-    /**Get the sub-type of the capability. This is a code dependent on the
-       main type of the capability.
-     */
-    virtual unsigned GetSubType() const;
-
-    /**Get the name of the media data format this class represents.
-     */
-    virtual PString GetFormatName() const;
-  //@}
-
-  protected:
-    Mode mode;
-};
-
-
-/**This class describes the G.723.1 codec capability.
- */
-class H323_G7231Capability : public H323AudioCapability
-{
-  PCLASSINFO(H323_G7231Capability, H323AudioCapability)
-
-  public:
-  /**@name Construction */
-  //@{
-    /**Create a new G.723.1 capability.
-     */
-    H323_G7231Capability(
-      PBoolean allowSIDFrames = PTrue   ///<  Allow SID frames in data stream.
-    );
-  //@}
-
-  /**@name Overrides from class PObject */
-  //@{
-    /**Create a copy of the object.
-      */
-    virtual PObject * Clone() const;
-  //@}
-
-  /**@name Identification functions */
-  //@{
-    /**Get the sub-type of the capability. This is a code dependent on the
-       main type of the capability.
-     */
-    virtual unsigned GetSubType() const;
-
-    /**Get the name of the media data format this class represents.
-     */
-    virtual PString GetFormatName() const;
-  //@}
-
-  /**@name Protocol manipulation */
-  //@{
-    /**This function is called whenever and outgoing TerminalCapabilitySet
-       or OpenLogicalChannel PDU is being constructed for the control channel.
-       It allows the capability to set the PDU fields from information in
-       members specific to the class.
-
-       The default behaviour sets the data rate field in the PDU.
-     */
-    virtual PBoolean OnSendingPDU(
-      H245_AudioCapability & pdu,  ///<  PDU to set information on
-      unsigned packetSize          ///<  Packet size to use in capability
-    ) const;
-
-    /**This function is called whenever and incoming TerminalCapabilitySet
-       or OpenLogicalChannel PDU has been used to construct the control
-       channel. It allows the capability to set from the PDU fields,
-       information in members specific to the class.
-
-       The default behaviour gets the data rate field from the PDU.
-     */
-    virtual PBoolean OnReceivedPDU(
-      const H245_AudioCapability & pdu,  ///<  PDU to get information from
-      unsigned & packetSize              ///<  Packet size to use in capability
-    );
-  //@}
-
-  protected:
-    PBoolean allowSIDFrames;
-};
-
-
-/**This class describes the GSM 06.10 codec capability.
- */
-class H323_GSM0610Capability : public H323AudioCapability
-{
-  PCLASSINFO(H323_GSM0610Capability, H323AudioCapability)
-
-  public:
-  /**@name Construction */
-  //@{
-    /**Create a new GSM 06.10 capability.
-     */
-    H323_GSM0610Capability();
-  //@}
-
-  /**@name Overrides from class PObject */
-  //@{
-    /**Create a copy of the object.
-      */
-    virtual PObject * Clone() const;
-  //@}
-
-  /**@name Identification functions */
-  //@{
-    /**Get the sub-type of the capability. This is a code dependent on the
-       main type of the capability.
-
-       This returns one of the four possible combinations of mode and speed
-       using the enum values of the protocol ASN H245_AudioCapability class.
-     */
-    virtual unsigned GetSubType() const;
-
-    /**Get the name of the media data format this class represents.
-     */
-    virtual PString GetFormatName() const;
-
-    /**Set the maximum size (in frames) of data that will be transmitted in a
-       single PDU.
-
-       This will also be the desired number that will be sent by most codec
-       implemetations.
-
-       The default behaviour sets the txFramesInPacket variable.
-     */
-    virtual void SetTxFramesInPacket(
-      unsigned frames   ///<  Number of frames per packet
-    );
-  //@}
-
-  /**@name Protocol manipulation */
-  //@{
-    /**This function is called whenever and outgoing TerminalCapabilitySet
-       or OpenLogicalChannel PDU is being constructed for the control channel.
-       It allows the capability to set the PDU fields from information in
-       members specific to the class.
-
-       The default behaviour sets the data rate field in the PDU.
-     */
-    virtual PBoolean OnSendingPDU(
-      H245_AudioCapability & pdu,  ///<  PDU to set information on
-      unsigned packetSize          ///<  Packet size to use in capability
-    ) const;
-
-    /**This function is called whenever and incoming TerminalCapabilitySet
-       or OpenLogicalChannel PDU has been used to construct the control
-       channel. It allows the capability to set from the PDU fields,
-       information in members specific to the class.
-
-       The default behaviour gets the data rate field from the PDU.
-     */
-    virtual PBoolean OnReceivedPDU(
-      const H245_AudioCapability & pdu,  ///<  PDU to get information from
-      unsigned & packetSize              ///<  Packet size to use in capability
-    );
-  //@}
-};
-
-#endif
-
-
 ///////////////////////////////////////////////////////////////////////////////
 
 /**This class describes the UserInput psuedo-channel.
