@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22640 $
+ * $Revision: 22745 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 03:19:20 +0000 (Tue, 19 May 2009) $
+ * $Date: 2009-05-28 01:04:54 +0000 (Thu, 28 May 2009) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -1746,6 +1746,15 @@ class H323Connection : public OpalRTPConnection
 
   /**@name Other services */
   //@{
+    /**Request a mode change to T.38 data.
+       Note this function is strictly H.323 and does operate correctly in the
+       OPAL media stream model. It is maintained for backward compatibility
+       with older applications only.
+      */
+    virtual PBoolean P_DEPRECATED RequestModeChangeT38(
+      const char * capabilityNames = "T.38\nT38FaxUDP"
+    );
+
     /**Get separate H.235 authentication for the connection.
        This allows an individual ARQ to override the authentical credentials
        used in H.235 based RAS for this particular connection.
@@ -2013,6 +2022,7 @@ class H323Connection : public OpalRTPConnection
     PBoolean transmitterSidePaused;
     bool     remoteTransmitPaused;
     PBoolean earlyStart;
+    PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
     PBoolean       endSessionNeeded;
     PSyncPoint endSessionReceived;
