@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22361 $
+ * $Revision: 23128 $
  * $Author: rjongbloed $
- * $Date: 2009-04-01 07:06:50 +0000 (Wed, 01 Apr 2009) $
+ * $Date: 2009-07-22 05:17:45 +0000 (Wed, 22 Jul 2009) $
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -168,6 +168,19 @@ class H323EndPoint : public OpalRTPEndPoint
     virtual void SetH221NonStandardInfo(
       H225_H221NonStandard & info
     ) const;
+
+    /**Set the Gateway supported protocol default always H.323
+      */
+    virtual bool SetGatewaySupportedProtocol(
+      H225_ArrayOf_SupportedProtocols & protocols
+    ) const;
+
+    /**Set the gateway prefixes 
+       Override this to set the acceptable prefixes to the gatekeeper
+      */
+    virtual bool OnSetGatewayPrefixes(
+      PStringList & prefixes
+    ) const;
   //@}
 
 
