@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21847 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-22 00:38:20 +0000 (Mon, 22 Dec 2008) $
  */
 
 #ifndef OPAL_H323_H323PDU_H
@@ -251,7 +251,7 @@ class H323SignalPDU : public H225_H323_UserInformation
       int screening = -1
     );
 
-#ifdef H323_H460
+#ifdef OPAL_H460
     /** When sending the H460 message in the Setup PDU you have to ensure
         the ARQ is received first then add the Fields to the Setup PDU
     	so we require a call back
@@ -471,20 +471,32 @@ PString H323GetAliasAddressE164(const H225_ArrayOf_AliasAddress & aliases);
 
 H323Connection::CallEndReason H323TranslateToCallEndReason(
   Q931::CauseValues cause,
-  const H225_ReleaseCompleteReason & reason
+  unsigned reason
 );
 Q931::CauseValues H323TranslateFromCallEndReason(
-  const H323Connection & connection,
-  H225_ReleaseCompleteReason & rcReason
+  H323Connection::CallEndReason callEndReason,
+  H225_ReleaseCompleteReason & releaseCompleteReason
 );
 
 void H323GetApplicationInfo(OpalProductInfo & info, const H225_VendorIdentifier & vendor);
 
+void H323SetRTPPacketization(
+  H245_ArrayOf_RTPPayloadType & rtpPacketizations,
+  PINDEX & packetizationsCount,
+  const OpalMediaFormat & mediaFormat,
+  RTP_DataFrame::PayloadTypes payloadType
+);
 bool H323SetRTPPacketization(
   H245_RTPPayloadType & rtpPacketization,
   const OpalMediaFormat & mediaFormat,
   RTP_DataFrame::PayloadTypes payloadType
 );
+bool H323SetRTPPacketization(
+  H245_RTPPayloadType & rtpPacketization,
+  const PString & mediaPacketizationString,
+  const OpalMediaFormat & mediaFormat,
+  RTP_DataFrame::PayloadTypes payloadType
+);
 PString H323GetRTPPacketization(
   const H245_RTPPayloadType & rtpPacketization
 );
