@@ -26,18 +26,19 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20626 $
+ * $Revision: 23148 $
  * $Author: rjongbloed $
- * $Date: 2008-07-26 08:27:01 +0200 (sam 26 jui 2008) $
+ * $Date: 2009-07-28 12:42:40 +0000 (Tue, 28 Jul 2009) $
  */
 
-#ifndef __LIDS_LIDEP_H
-#define __LIDS_LIDEP_H
+#ifndef OPAL_LIDS_LIDEP_H
+#define OPAL_LIDS_LIDEP_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 #include <opal/endpoint.h>
 #include <lids/lid.h>
@@ -482,6 +483,12 @@ class OpalLineConnection : public OpalConnection
   //@}
 
 
+  /**@name Member variable access */
+  //@{
+    /**Get the line being used by this media stream.
+      */
+    OpalLine & GetLine() { return line; }
+
     /** delay in msec to wait between the dial tone detection and dialing the dtmf 
       * @param uiDial the dial delay to set
      */
@@ -491,7 +498,7 @@ class OpalLineConnection : public OpalConnection
      * @return uiDialDelay the dial delay to get
      */
     unsigned int getDialDelay() const { return m_dialParams.m_dialStartDelay; }
-
+  //@}
         
   protected:
     OpalLineEndPoint & endpoint;
@@ -582,7 +589,8 @@ class OpalLineMediaStream : public OpalMediaStream
        The default behaviour does nothing.
       */
     virtual PBoolean SetDataSize(
-      PINDEX dataSize  ///<  New data size
+      PINDEX dataSize,  ///< New data size (in total)
+      PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
     /**Indicate if the media stream is synchronous.
@@ -657,7 +665,7 @@ class OpalLineSilenceDetector : public OpalSilenceDetector
 };
 
 
-#endif // __LIDS_LIDEP_H
+#endif // OPAL_LIDS_LIDEP_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
