@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22527 $
+ * $Revision: 23153 $
  * $Author: rjongbloed $
- * $Date: 2009-05-06 08:58:09 +0000 (Wed, 06 May 2009) $
+ * $Date: 2009-07-29 02:22:00 +0000 (Wed, 29 Jul 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -304,7 +304,7 @@ class OpalMediaOption : public PObject
     };
 
     const H245GenericInfo & GetH245Generic() const { return m_H245Generic; }
-    void SetH245Generic(const H245GenericInfo & generic) { m_H245Generic = generic; }
+    void SetH245Generic(const H245GenericInfo & genericInfo) { m_H245Generic = genericInfo; }
 #endif // OPAL_H323
 
   protected:
@@ -446,6 +446,7 @@ class OpalMediaOptionEnum : public OpalMediaOption
     PINDEX GetValue() const { return m_value; }
     void SetValue(PINDEX value);
 
+    const PStringArray & GetEnumerations() const { return m_enumerations; }
     void SetEnumerations(const PStringArray & e)
     {
       m_enumerations = e;
