@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22089 $
+ * $Revision: 23148 $
  * $Author: rjongbloed $
- * $Date: 2009-02-19 04:19:55 +0000 (Thu, 19 Feb 2009) $
+ * $Date: 2009-07-28 12:42:40 +0000 (Tue, 28 Jul 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -228,7 +228,8 @@ class OpalMediaStream : public PSafeObject
        The default behaviour does nothing.
       */
     virtual PBoolean SetDataSize(
-      PINDEX dataSize  ///<  New data size
+      PINDEX dataSize,  ///< New data size (in total)
+      PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
     /**Get the data size in bytes that is expected to be used. Some media
@@ -317,7 +318,7 @@ class OpalMediaStream : public PSafeObject
 
     /**Returns true if the media stream is open.
       */
-    bool IsOpen() { return isOpen; }
+    bool IsOpen() const { return isOpen; }
     
     /**Set the patch thread that is using this stream.
       */
@@ -346,7 +347,7 @@ class OpalMediaStream : public PSafeObject
     bool RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
 
 #if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
 #endif
   //@}
 
@@ -508,7 +509,8 @@ class OpalRTPMediaStream : public OpalMediaStream
     /**Set the data size in bytes that is expected to be used.
       */
     virtual PBoolean SetDataSize(
-      PINDEX dataSize  ///<  New data size
+      PINDEX dataSize,  ///< New data size (in total)
+      PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
     /**Indicate if the media stream is synchronous.
@@ -528,7 +530,7 @@ class OpalRTPMediaStream : public OpalMediaStream
     { return rtpSession; }
 
 #if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
 #endif
   //@}
 
@@ -726,7 +728,8 @@ class OpalAudioMediaStream : public OpalRawMediaStream
        The defafault simply sets teh member variable defaultDataSize.
       */
     virtual PBoolean SetDataSize(
-      PINDEX dataSize  ///<  New data size
+      PINDEX dataSize,  ///< New data size (in total)
+      PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
     /**Indicate if the media stream is synchronous.
@@ -816,7 +819,8 @@ class OpalVideoMediaStream : public OpalMediaStream
     /** Override size of frame header is included
       */
     virtual PBoolean SetDataSize(
-     PINDEX dataSize  ///<  New data size
+      PINDEX dataSize,  ///< New data size (in total)
+      PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
     /** Get the input device (e.g. for statistics)
