@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21939 $
- * $Author: rjongbloed $
- * $Date: 2009-01-19 23:50:29 +0000 (Mon, 19 Jan 2009) $
+ * $Revision: 22531 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-06 13:12:26 +0000 (Wed, 06 May 2009) $
  */
 
 #ifndef OPAL_OPAL_RTPCONN_H
@@ -124,7 +124,7 @@ class OpalRTPMediaSession : public OpalMediaSession
 
     virtual bool IsRTP() const { return true; }
 
-    virtual bool HasFailed() const { return rtpSession != NULL && rtpSession->HasFailed(); }
+    virtual bool HasFailed() const { return (rtpSession != NULL) && (rtpSession->HasFailed() || (rtpSession->GetPacketsReceived() == 0)); }
 
     virtual OpalTransportAddress GetLocalMediaAddress() const;
 
