@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22178 $
+ * $Revision: 23165 $
  * $Author: rjongbloed $
- * $Date: 2009-03-09 10:04:21 +0000 (Mon, 09 Mar 2009) $
+ * $Date: 2009-07-29 04:57:29 +0000 (Wed, 29 Jul 2009) $
  */
 
 #ifndef OPAL_T38_T38PROTO_H
@@ -91,6 +91,7 @@ class OpalFaxCallInfo {
     OpalFaxCallInfo();
     PUDPSocket socket;
     PPipeChannel spanDSP;
+    PThread * stdoutThread;
     unsigned refCount;
     PIPSocket::Address spanDSPAddr;
     WORD spanDSPPort;
@@ -153,11 +154,16 @@ class OpalFaxMediaStream : public OpalMediaStream
       */
     virtual PBoolean IsSynchronous() const;
 
-    virtual PString GetSpanDSPCommandLine(OpalFaxCallInfo &);
-
+#if OPAL_STATISTICS
+    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
+#endif
   //@}
 
+    virtual PString GetSpanDSPCommandLine(OpalFaxCallInfo &);
+
   protected:
+    PDECLARE_NOTIFIER(PThread, OpalFaxMediaStream, ReadStdOut);
+
     OpalFaxConnection & m_connection;
     PMutex              infoMutex;
     PString             sessionToken;
@@ -167,6 +173,12 @@ class OpalFaxMediaStream : public OpalMediaStream
     BYTE                writeBuffer[320];
     PINDEX              writeBufferLen;
     PString             m_stationId;
+
+#if OPAL_STATISTICS
+    OpalMediaStatistics::Fax m_statistics;
+#else
+    int m_result;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -263,6 +275,14 @@ class OpalFaxEndPoint : public OpalEndPoint
     virtual void AcceptIncomingConnection(
       const PString & connectionToken ///<  Token of connection to accept call
     );
+
+    /**Fax transmission/receipt completed.
+       Default behaviour releases the connection.
+      */
+    virtual void OnFaxCompleted(
+      OpalFaxConnection & connection, ///< Connection that completed.
+      bool failed   ///< Fax ended with failure
+    );
   //@}
 
   /**@name Member variable access */
@@ -376,19 +396,23 @@ class OpalFaxConnection : public OpalConnection
       */
     virtual void AcceptIncoming();
 
-    /** Check for no more I/O which means fax stopped.
+    /**Fax transmission/receipt completed.
+       Default behaviour calls equivalent function on OpalFaxEndPoint.
       */
-    void CheckFaxStopped();
+    virtual void OnFaxCompleted(
+      bool failed   ///< Fax ended with failure
+    );
+
+    /**Get receive fax flag.
+      */
+    bool IsReceive() const { return m_receive; }
   //@}
 
   protected:
-    PDECLARE_NOTIFIER(PTimer,  OpalFaxConnection, OnFaxStoppedTimeout);
-
     OpalFaxEndPoint & m_endpoint;
     PString           m_filename;
     bool              m_receive;
     PString           m_stationId;
-    PTimer            m_faxStopped;
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -416,7 +440,7 @@ class OpalT38Connection : public OpalFaxConnection
     virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
     virtual void OnEstablished();
     virtual OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
-    virtual void OnClosedMediaStream(const OpalMediaStream & stream);
+    virtual void OnMediaPatchStop(unsigned sessionId, bool isSource);
     virtual OpalMediaFormatList GetMediaFormats() const;
 
     // triggers into fax mode
