@@ -103,6 +103,19 @@ static struct StdSizes {
 
 static FFMPEGLibrary FFMPEGLibraryInstance(CODEC_ID_H263P);
 
+__inline unsigned char * malloc_aligned(size_t size)
+{
+#if HAVE_POSIX_MEMALIGN
+  void * buffer;
+  if (posix_memalign(&buffer, 64, size) != 0)
+    return NULL;
+  return (unsigned char *)buffer;
+#else
+  return (unsigned char *)malloc(size);
+#endif
+}
+
+
 static void logCallbackFFMPEG (void* v, int level, const char* fmt , va_list arg)
 {
   char buffer[512];
@@ -119,13 +132,10 @@ static void logCallbackFFMPEG (void* v, int level, const char* fmt , va_list arg
     vsprintf(buffer + strlen(buffer), fmt, arg);
     if (strlen(buffer) > 0)
       buffer[strlen(buffer)-1] = 0;
-printf("%s\n", buffer);
-/*
-    if (severity = 4) 
+    if (severity == 4) 
       { TRACE_UP (severity, buffer); }
     else
       { TRACE (severity, buffer); }
-*/
   }
 }
 
@@ -267,6 +277,9 @@ bool H263_Base_EncoderContext::Open(CodecID codecId)
   _context->flags |= CODEC_FLAG_EMU_EDGE;        // don't draw edges
   _context->flags |= CODEC_FLAG_PASS1;
 
+  _context->error_concealment = 3;
+  _context->error_recognition = 5;
+
   // debugging flags
   if (Trace::CanTraceUserPlane(4)) {
     _context->debug |= FF_DEBUG_RC;
@@ -653,12 +666,7 @@ int H263_RFC2190_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
 
     if (_inputFrameBuffer != NULL)
       free(_inputFrameBuffer);
-#if HAVE_POSIX_MEMALIGN
-    if (posix_memalign((void **)&_inputFrameBuffer, 64, header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2)) != 0) 
-#else
-    if ((_inputFrameBuffer = (unsigned char *)malloc(header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2))) != NULL) 
-#endif
-    {
+    if ((_inputFrameBuffer = malloc_aligned(header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2))) == NULL) {
       TRACE_AND_LOG(tracer, 1, "Unable to allocate memory for frame buffer");
       return 0;
     }
@@ -697,12 +705,7 @@ int H263_RFC2190_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
   }
   if (packetizer.m_buffer == NULL) {
     packetizer.m_bufferSize = newOutputSize;
-#if HAVE_POSIX_MEMALIGN
-    if (posix_memalign((void **)&packetizer.m_buffer, 64, packetizer.m_bufferSize) != 0) 
-#else
-    if ((packetizer.m_buffer = (unsigned char *)malloc(packetizer.m_bufferSize)) != NULL) 
-#endif
-    {
+    if ((packetizer.m_buffer = malloc_aligned(packetizer.m_bufferSize)) == NULL) {
       TRACE_AND_LOG(tracer, 1, "Unable to allocate memory for packet buffer");
       return 0;
     }
@@ -861,11 +864,7 @@ int H263_RFC2429_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLe
     }
     if (_inputFrameBuffer != NULL)
       free(_inputFrameBuffer);
-#if HAVE_POSIX_MEMALIGN
-    if (posix_memalign((void **)&_inputFrameBuffer, 64, header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2)) != 0) {
-#else
-    if ((_inputFrameBuffer = (unsigned char *)malloc(header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2))) != NULL) {
-#endif
+    if ((_inputFrameBuffer = malloc_aligned(header->width*header->height*3/2 + (FF_INPUT_BUFFER_PADDING_SIZE*2))) == NULL) {
       TRACE_AND_LOG(tracer, 1, "Unable to allocate memory for frame buffer");
       return 0;
     }
@@ -1189,7 +1188,7 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
   bool requestIFrame, isIFrame;
   int code = depacketizer.SetPacket(srcRTP, requestIFrame, isIFrame);
   if (code <= 0) {
-    flags = 0;
+    flags = requestIFrame ? PluginCodec_ReturnCoderRequestIFrame : 0;
     return ReturnEmptyFrame(dstRTP, dstLen, flags);
   }
 
@@ -1204,6 +1203,10 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
 
   TRACE_AND_LOG(tracer, 4, "Decoder called with " << depacketizer.frame.size()  << " bytes");
 
+#if FFMPEG_HAS_DECODE_ERROR_COUNT
+  unsigned error_before = _context->decode_error_count;
+#endif
+
   int gotPicture = 0;
   int bytesDecoded = FFMPEGLibraryInstance.AvcodecDecodeVideo(_context, _outputFrame, &gotPicture, &depacketizer.frame[0], depacketizer.frame.size());
 
@@ -1218,7 +1221,11 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
   TRACE_AND_LOG(tracer, 4, "Decoder processed " << bytesDecoded << " bytes, creating frame at " << _context->width << "x" << _context->height);
 
   // if error occurred, tell the other end to send another I-frame and hopefully we can resync
-  if (bytesDecoded < 0) {
+  if (bytesDecoded < 0
+#if FFMPEG_HAS_DECODE_ERROR_COUNT
+      || error_before != _context->decode_error_count
+#endif
+  ) {
     flags = PluginCodec_ReturnCoderRequestIFrame;
     return ReturnEmptyFrame(dstRTP, dstLen, flags);
   }
@@ -1253,7 +1260,7 @@ bool H263_RFC2190_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcL
   int size = _context->width * _context->height;
 
   if (dstRTP.GetPayloadSize() < (frameBytes + sizeof(PluginCodec_Video_FrameHeader))) {
-    flags = 0;
+    flags = PluginCodec_ReturnCoderRequestIFrame;
     TRACE_AND_LOG(tracer, 1, "Destination buffer " << dstLen << " insufficient for decoded data size " << header->width << "x" << header->height);
     return ReturnEmptyFrame(dstRTP, dstLen, flags);
   }
