@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21388 $
+ * $Revision: 22617 $
  * $Author: csoutheren $
- * $Date: 2008-10-20 04:45:50 +0000 (Mon, 20 Oct 2008) $
+ * $Date: 2009-05-14 03:59:47 +0000 (Thu, 14 May 2009) $
  */
 
 #ifndef _CodecTest_MAIN_H
@@ -32,19 +32,19 @@
 #include <codec/vidcodec.h>
 #include <opal/patch.h>
 
-
 class TranscoderThread : public PThread
 {
   public:
     TranscoderThread(unsigned _num, const char * name)
       : PThread(5000, NoAutoDeleteThread, NormalPriority, name)
-      , running(true)
+      , running(false)
       , encoder(NULL)
       , decoder(NULL)
       , num(_num)
       , timestamp(0)
       , markerHandling(NormalMarkers)
-      , rcEnable(false)
+      , rateController(NULL)
+      , m_dropPercent(0)
     {
     }
 
@@ -60,6 +60,7 @@ class TranscoderThread : public PThread
 
     virtual bool Read(RTP_DataFrame & frame) = 0;
     virtual bool Write(const RTP_DataFrame & frame) = 0;
+    void Start() { if (running) Resume(); }
     virtual void Stop() = 0;
 
     virtual void UpdateStats(const RTP_DataFrame &) { }
@@ -86,13 +87,13 @@ class TranscoderThread : public PThread
     } markerHandling;
 
     PDECLARE_NOTIFIER(OpalMediaCommand, TranscoderThread, OnTranscoderCommand);
-    bool forceIFrame;
+    bool m_forceIFrame;
 
-    bool rcEnable;
-    OpalVideoRateController rateController;
+    OpalVideoRateController * rateController;
     int framesToTranscode;
     int frameTime;
     bool calcSNR;
+    int m_dropPercent;
 };
 
 
@@ -135,7 +136,6 @@ class VideoThread : public TranscoderThread
       , singleStep(false)
       , frameWait(0, INT_MAX)
     {
-      InitStats();
     }
 
     ~VideoThread()
@@ -151,9 +151,8 @@ class VideoThread : public TranscoderThread
     virtual bool Write(const RTP_DataFrame & frame);
     virtual void Stop();
 
-    void InitStats();
-    virtual void UpdateStats(const RTP_DataFrame &);
-    void UpdateFrameStats();
+    void CalcVideoPacketStats(const RTP_DataFrameList & frames, bool isIFrame);
+    void WriteFrameStats(const PString & str);
 
     PVideoInputDevice  * grabber;
     PVideoOutputDevice * display;
@@ -162,23 +161,26 @@ class VideoThread : public TranscoderThread
     PSemaphore           frameWait;
     unsigned             frameRate;
 
+    void SaveSNRFrame(const RTP_DataFrame * src)
+    { snrSourceFrames.push(src); }
+
+    void CalcSNR(const RTP_DataFrame & src);
     void CalcSNR(const RTP_DataFrame & src, const RTP_DataFrame & dst);
     void ReportSNR();
 
     PString frameFn;
-    PString packetFn;
-
-    PTextFile packetStatFile, frameStatFile;
-    PInt64 packetCount;
+    PTextFile frameStatFile;
     PInt64 frameCount;
-    PInt64 frameBytes;
-    PInt64 totalFrameBytes;
+    DWORD frameStartTimestamp;
+    std::queue<const RTP_DataFrame *> snrSourceFrames;
 
     unsigned snrWidth, snrHeight;
     double sumYSNR;
     double sumCbSNR;
     double sumCrSNR;
     PInt64 snrCount;
+
+    OpalBitRateCalculator m_bitRateCalc;
 };
 
 
@@ -192,10 +194,20 @@ class CodecTest : public PProcess
 
     virtual void Main();
 
-    class TestThreadInfo {
+    class TestThreadInfo : public PObject
+    {
       public:
-        TestThreadInfo(unsigned _num)
-          : audio(_num), video(_num) { }
+        TestThreadInfo(unsigned num)
+          : audio(num)
+          , video(num)
+        {
+        }
+
+        bool Initialise(PArgList & args) { return audio.Initialise(args) && video.Initialise(args); }
+        void Start() { audio.Start(); video.Start(); }
+        void Stop()  { audio.Stop(); video.Stop(); }
+        void Wait()  { audio.WaitForTermination(); video.WaitForTermination(); }
+
         AudioThread audio;
         VideoThread video;
     };
