@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21859 $
+ * $Revision: 22851 $
  * $Author: rjongbloed $
- * $Date: 2008-12-22 04:30:11 +0000 (Mon, 22 Dec 2008) $
+ * $Date: 2009-06-12 10:17:45 +0000 (Fri, 12 Jun 2009) $
  */
 
 #include "precompile.h"
@@ -145,7 +145,7 @@ void IvrOPAL::Main()
   // Set up SIP
   interfaces = args.GetOptionString('S');
   if (interfaces != "x") {
-    SIPEndPoint * sip  = new SIPEndPoint(*m_manager);
+    MySIPEndPoint * sip  = new MySIPEndPoint(*m_manager);
     if (!sip->StartListeners(interfaces.Lines())) {
       cerr << "Could not start SIP listeners." << endl;
       return;
@@ -161,7 +161,17 @@ void IvrOPAL::Main()
       params.m_expire = 300;
 
       PString aor;
-      sip->Register(params, aor);
+      if (!sip->Register(params, aor)) {
+        cerr << "Could not start SIP registration to " << params.m_addressOfRecord << endl;
+        return;
+      }
+
+      sip->m_completed.Wait();
+
+      if (!sip->IsRegistered(aor)) {
+        cerr << "Could not complete SIP registration for " << aor << endl;
+        return;
+      }
     }
 
     m_manager->AddRouteEntry("sip.*:.* = ivr:");
@@ -208,6 +218,10 @@ void IvrOPAL::Main()
   // Wait for call to come in and finish
   m_manager->m_completed.Wait();
   cout << " completed.";
+
+  MyManager * mgr = m_manager;
+  m_manager = NULL;
+  delete mgr;
 }
 
 
@@ -228,4 +242,12 @@ bool IvrOPAL::OnInterrupt(bool)
 }
 
 
+void MySIPEndPoint::OnRegistrationStatus(const RegistrationStatus & status)
+{
+  SIPEndPoint::OnRegistrationStatus(status);
+  if (status.m_reason >= SIP_PDU::Successful_OK)
+    m_completed.Signal();
+}
+
+
 // End of File ///////////////////////////////////////////////////////////////
