@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21655 $
+ * $Revision: 23491 $
  * $Author: rjongbloed $
- * $Date: 2008-11-30 05:16:47 +0000 (Sun, 30 Nov 2008) $
+ * $Date: 2009-09-18 02:17:05 +0000 (Fri, 18 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -574,7 +574,7 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
   setup.IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
   {
     PString callingParty;
-    callingParty = connection.GetStringOptions()("Calling-Party-Name");
+    callingParty = connection.GetStringOptions()(OPAL_OPT_CALLING_PARTY_NAME);
     if (callingParty.IsEmpty())
       H323SetAliasAddresses(endpoint.GetAliasNames(), setup.m_sourceAddress);
     else
@@ -832,6 +832,12 @@ Q931::CauseValues H323TranslateFromCallEndReason(H323Connection::CallEndReason c
     Q931::TemporaryFailure,                                 /// EndedByTemporaryFailure   The remote failed temporarily app may retry
     Q931::UnknownCauseIE,                                   /// EndedByQ931Cause,         The remote ended the call with unmapped Q.931 cause code
     Q931::NormalUnspecified,                                /// EndedByDurationLimit,     Call cleared due to an enforced duration limit
+    Q931::InvalidCallReference,                             /// EndedByInvalidConferenceID,  Call cleared due to invalid conference ID
+    Q931::NoResponse,                                       /// EndedByNoDialTone,           Call cleared due to missing dial tone
+    Q931::NoResponse,                                       /// EndedByNoRingBackTone,       Call cleared due to missing ringback tone
+    Q931::DestinationOutOfOrder,                            /// EndedByOutOfService,         Call cleared because the line is out of service, 
+    Q931::Redirection,                                      /// EndedByAcceptingCallWaiting, Call cleared because another call is answered
+    Q931::ExchangeRoutingError,                             /// EndedByGkAdmissionFailed,    Call cleared because gatekeeper admission request failed.
   };
 
   int code = ReasonCodes[callEndReason];
@@ -1230,23 +1236,8 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
 
   {
     const OpalConnection::StringOptions & stringOptions = connection.GetStringOptions();
-    PString strLocal, strDisplay;
-    strLocal   = stringOptions("Calling-Party-Name");
-    strDisplay = stringOptions("Calling-Display-Name");
-    if (strDisplay.IsEmpty())
-      strDisplay = strLocal;
-
-    if (!strLocal.IsEmpty())
-      localName = strLocal;
-    else
-      localName = connection.GetLocalPartyName();
-
-    if (!strDisplay.IsEmpty())
-      displayName = strDisplay;
-    else if (!strLocal.IsEmpty())
-      displayName = localName;
-    else
-      displayName = connection.GetDisplayName();
+    localName   = stringOptions(OPAL_OPT_CALLING_PARTY_NAME, connection.GetLocalPartyName());
+    displayName = stringOptions(OPAL_OPT_CALLING_DISPLAY_NAME, connection.GetDisplayName());
   }
 
   if (IsE164(localName)) {
@@ -1261,7 +1252,7 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
     }
   }
   else {
-    if (!localName && displayName.IsEmpty())
+    if (displayName.IsEmpty())
       displayName = localName;
     for (PStringList::const_iterator alias = aliases.begin(); alias != aliases.end(); ++alias) {
       if (IsE164(*alias)) {
@@ -1271,8 +1262,6 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
     }
   }
 
-  if (displayName.IsEmpty())
-    displayName = number;
   q931pdu.SetDisplayName(displayName);
 
   if (insertPartyNumbers) {
