@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22096 $
+ * $Revision: 23294 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2009-08-28 06:26:26 +0000 (Fri, 28 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -554,13 +554,13 @@ PBoolean OpalLineConnection::SetConnected()
 
   if (line.IsTerminal()) {
     if (!line.SetConnected()) {
-      PTRACE(1, "LID Con\tCould set line to connected mode on " << *this);
+      PTRACE(1, "LID Con\tCould not set line to connected mode on " << *this);
       return false;
     }
   }
   else {
     if (!line.SetOffHook()) {
-      PTRACE(1, "LID Con\tCould set line off hook on " << *this);
+      PTRACE(1, "LID Con\tCould not set line off hook on " << *this);
       return false;
     }
     PTRACE(4, "LID Con\tAnswered call - gone off hook.");
@@ -936,7 +936,7 @@ PBoolean OpalLineMediaStream::Open()
       return false;
   }
 
-  SetDataSize(GetDataSize());
+  SetDataSize(GetDataSize(), GetDataSize()/2);
   PTRACE(3, "LineMedia\tStream opened for " << mediaFormat << ", using "
          << (notUsingRTP ? (useDeblocking ? "reblocked audio" : "audio frames") : "direct RTP"));
 
@@ -1105,7 +1105,7 @@ PBoolean OpalLineMediaStream::WriteData(const BYTE * buffer, PINDEX length, PIND
 }
 
 
-PBoolean OpalLineMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalLineMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
   if (notUsingRTP) {
     if (IsSource())
@@ -1119,7 +1119,8 @@ PBoolean OpalLineMediaStream::SetDataSize(PINDEX dataSize)
            << (useDeblocking ? "needs" : "no") << " reblocking.");
   }
 
-  return OpalMediaStream::SetDataSize(dataSize);
+  // Yes we set BOTH to frameSize, and ignore the dataSize parameter
+  return OpalMediaStream::SetDataSize(dataSize, frameTime);
 }
 
 
