@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22164 $
- * $Author: rjongbloed $
- * $Date: 2009-03-06 00:23:42 +0000 (Fri, 06 Mar 2009) $
+ * $Revision: 22465 $
+ * $Author: csoutheren $
+ * $Date: 2009-04-24 18:24:46 +0000 (Fri, 24 Apr 2009) $
  */
 
 #include <ptlib.h>
@@ -353,6 +353,7 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection,
   PSafePtr<OpalConnection> otherConnection;
   while (EnumerateConnections(otherConnection, PSafeReadOnly, includeSpecifiedConnection ? NULL : &connection)) {
     OpalMediaFormatList possibleFormats = OpalTranscoder::GetPossibleFormats(otherConnection->GetMediaFormats());
+    otherConnection->AdjustMediaFormats(possibleFormats);
     if (first) {
       commonFormats = possibleFormats;
       first = PFalse;
@@ -382,7 +383,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
                                                   unsigned sessionID, 
                                    const OpalMediaFormat & preselectedFormat)
 {
-  PSafeLockReadOnly lock(*this);
+  PSafeLockReadWrite lock(*this);
   if (isClearing || !lock.IsLocked())
     return false;
 
