@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22563 $
- * $Author: csoutheren $
- * $Date: 2009-05-07 17:34:54 +0000 (Thu, 07 May 2009) $
+ * $Revision: 23153 $
+ * $Author: rjongbloed $
+ * $Date: 2009-07-29 02:22:00 +0000 (Wed, 29 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -1087,6 +1087,10 @@ static bool SetOptionOfType(OpalMediaFormatInternal & format, const PString & na
 bool OpalMediaFormatInternal::GetOptionBoolean(const PString & name, bool dflt) const
 {
   PWaitAndSignal m(media_format_mutex);
+  const OpalMediaOptionEnum * optEnum = dynamic_cast<const OpalMediaOptionEnum *>(FindOption(name));
+  if (optEnum != NULL && optEnum->GetEnumerations().GetSize() == 2)
+    return optEnum->GetValue() != 0;
+
   return GetOptionOfType<OpalMediaOptionBoolean, bool>(*this, name, dflt);
 }
 
@@ -1094,6 +1098,12 @@ bool OpalMediaFormatInternal::GetOptionBoolean(const PString & name, bool dflt)
 bool OpalMediaFormatInternal::SetOptionBoolean(const PString & name, bool value)
 {
   PWaitAndSignal m(media_format_mutex);
+  OpalMediaOptionEnum * optEnum = dynamic_cast<OpalMediaOptionEnum *>(FindOption(name));
+  if (optEnum != NULL && optEnum->GetEnumerations().GetSize() == 2) {
+    optEnum->SetValue(value);
+    return true;
+  }
+
   return SetOptionOfType<OpalMediaOptionBoolean, bool>(*this, name, value);
 }
 
