@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 22640 $
+ * $Revision: 23149 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 03:19:20 +0000 (Tue, 19 May 2009) $
+ * $Date: 2009-07-29 00:37:30 +0000 (Wed, 29 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -392,7 +392,7 @@ PBoolean OpalMediaStream::PushPacket(RTP_DataFrame & packet)
 }
 
 
-PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize, PINDEX /*frameTime*/)
 {
   if (dataSize <= 0)
     return false;
@@ -661,11 +661,8 @@ PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 }
 
 
-PBoolean OpalRTPMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /*frameTime*/)
 {
-  if (dataSize <= 0)
-    return false;
-
   PTRACE(3, "Media\tRTP data size cannot be changed to " << dataSize << ", fixed at " << defaultDataSize);
   return true;
 }
@@ -738,13 +735,21 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
   if (!IsOpen() || m_channel == NULL)
     return false;
 
-  if (!m_channel->Read(buffer, size))
-    return false;
+  if (buffer == NULL || size == 0)
+    return m_channel->Read(buffer, size);
 
-  length = m_channel->GetLastReadCount();
+  while (size > 0) {
+    if (!m_channel->Read(buffer, size))
+      return false;
+
+    PINDEX lastReadCount = m_channel->GetLastReadCount();
+    CollectAverage(buffer, lastReadCount);
+
+    buffer += lastReadCount;
+    length += lastReadCount;
+    size -= lastReadCount;
+  }
 
-  if (buffer != NULL)
-    CollectAverage(buffer, length);
 
   return true;
 }
@@ -812,7 +817,7 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
   m_channel = chan;
   m_autoDelete = autoDelete;
 
-  SetDataSize(GetDataSize());
+  SetDataSize(GetDataSize(), 1);
 
   m_channelMutex.Signal();
 
@@ -954,22 +959,27 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
 }
 
 
-PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
   /* For efficiency reasons we will not accept a packet size that is too small.
      We move it up to the next even multiple, which has a danger of the remote not
      sending an even number of our multiplier. */
-  const unsigned MinTime = 10;
-  PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTime)*sizeof(short);
-  if (dataSize < minSize) {
-    PTRACE(1, "Media\tClamping audio stream data size from " << dataSize << " to minimum " << minSize);
-    dataSize = ((minSize+dataSize-1)/dataSize)*dataSize;
-  }
+  const unsigned BufferTimeMilliseconds = 10;
+  PINDEX bufferSize = BufferTimeMilliseconds*mediaFormat.GetClockRate()/1000*sizeof(short);
+
+  // Quantise dataSize up to multiple of the frame time
+  PINDEX frameSize = frameTime*sizeof(short);
+  dataSize = (dataSize+frameSize-1)/frameSize * frameSize;
+
+  // Calculte buffers needed for the data we require and increase buffers if required
+  PINDEX dataBuffersNeeded = (dataSize+bufferSize-1)/bufferSize;
+  if (dataBuffersNeeded < soundChannelBuffers)
+    dataBuffersNeeded = soundChannelBuffers;
 
   PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
-         << dataSize << " bytes and " << soundChannelBuffers << " buffers.");
-  return OpalMediaStream::SetDataSize(dataSize) &&
-         ((PSoundChannel *)m_channel)->SetBuffers(dataSize, soundChannelBuffers);
+         << dataSize << ", buffer size set to " << bufferSize << " and " << dataBuffersNeeded << " buffers.");
+  return OpalMediaStream::SetDataSize(dataSize, frameTime) &&
+         ((PSoundChannel *)m_channel)->SetBuffers(bufferSize, dataBuffersNeeded);
 }
 
 
@@ -1010,7 +1020,7 @@ OpalVideoMediaStream::~OpalVideoMediaStream()
 }
 
 
-PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
   if (inputDevice != NULL) {
     PINDEX minDataSize = inputDevice->GetMaxFrameBytes();
@@ -1023,7 +1033,7 @@ PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize)
       dataSize = minDataSize;
   }
 
-  return OpalMediaStream::SetDataSize(sizeof(PluginCodec_Video_FrameHeader) + dataSize); 
+  return OpalMediaStream::SetDataSize(sizeof(PluginCodec_Video_FrameHeader) + dataSize, frameTime);
 }
 
 
@@ -1062,7 +1072,7 @@ PBoolean OpalVideoMediaStream::Open()
     }
   }
 
-  SetDataSize(0); // Gets set to minimum of device buffer requirements
+  SetDataSize(1, 1); // Gets set to minimum of device buffer requirements
 
   return OpalMediaStream::Open();
 }
