@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21159 $
+ * $Revision: 21833 $
  * $Author: rjongbloed $
- * $Date: 2008-09-25 03:12:55 +0000 (Thu, 25 Sep 2008) $
+ * $Date: 2008-12-18 05:15:31 +0000 (Thu, 18 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -82,11 +82,18 @@ OpalMediaTypeDefinition * OpalMediaType::GetDefinition(const OpalMediaType & key
 
 ///////////////////////////////////////////////////////////////////////////////
 
+OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * mediaType,
 #if OPAL_SIP
-OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * _mediaType, const char * _sdpType, unsigned preferredSessionId)
-  : mediaType(_mediaType), sdpType(_sdpType != NULL ? _sdpType : "")
+                                                 const char * sdpType,
 #else
-OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * _mediaType, const char *         , unsigned preferredSessionId)
+                                                 const char *,
+#endif
+                                                 unsigned preferredSessionId,
+                                                 OpalMediaType::AutoStartMode autoStart)
+  : m_mediaType(mediaType)
+  , m_autoStart(autoStart)
+#if OPAL_SIP
+  , m_sdpType(sdpType != NULL ? sdpType : "")
 #endif
 {
   PWaitAndSignal m(GetMapMutex());
@@ -104,10 +111,17 @@ OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * _mediaType, const
       ++preferredSessionId;
   }
 
-  typeMap.insert(SessionIDToMediaTypeMap_T::value_type(preferredSessionId, _mediaType));
-  GetMediaTypeToSessionIDMap().insert(MediaTypeToSessionIDMap_T::value_type(_mediaType, preferredSessionId));
+  typeMap.insert(SessionIDToMediaTypeMap_T::value_type(preferredSessionId, mediaType));
+  GetMediaTypeToSessionIDMap().insert(MediaTypeToSessionIDMap_T::value_type(mediaType, preferredSessionId));
 }
 
+
+OpalMediaSession * OpalMediaTypeDefinition::CreateMediaSession(OpalConnection & /*conn*/, unsigned /* sessionID*/) const
+{ 
+  return NULL; 
+}
+
+
 unsigned OpalMediaTypeDefinition::GetDefaultSessionId(const OpalMediaType & mediaType)
 {
   PWaitAndSignal m(GetMapMutex());
@@ -139,6 +153,7 @@ OpalMediaTypeDefinition::SessionIDToMediaTypeMap_T & OpalMediaTypeDefinition::Ge
   return map;
 }
 
+
 OpalMediaTypeDefinition::MediaTypeToSessionIDMap_T & OpalMediaTypeDefinition::GetMediaTypeToSessionIDMap()
 {
   PWaitAndSignal m(GetMapMutex());
@@ -154,33 +169,27 @@ PMutex & OpalMediaTypeDefinition::GetMapMutex()
 }
 
 
-RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & connection,
-#if OPAL_RTP_AGGREGATE
-                                                    PHandleAggregator * agg,
-#endif
-                                                    OpalSecurityMode * securityMode,
+RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & /*connection*/,
                                                     unsigned sessionID,
                                                     bool remoteIsNAT)
 {
   RTP_Session::Params params;
   params.id = sessionID;
   params.encoding = GetRTPEncoding();
-#if OPAL_RTP_AGGREGATE
-  params.aggregator = agg;
-#endif
-  params.isAudio = mediaType == OpalMediaType::Audio();
+  params.isAudio = m_mediaType == OpalMediaType::Audio();
   params.remoteIsNAT = remoteIsNAT;
 
-  if (securityMode != NULL) 
-    return securityMode->CreateRTPSession(connection, params);
-
   return new RTP_UDP(params);
 }
 
+
 ///////////////////////////////////////////////////////////////////////////////
 
-OpalRTPAVPMediaType::OpalRTPAVPMediaType(const char * mediaType, const char * sdpType, unsigned preferredSessionId)
-  : OpalMediaTypeDefinition(mediaType, sdpType, preferredSessionId)
+OpalRTPAVPMediaType::OpalRTPAVPMediaType(const char * mediaType,
+                                         const char * sdpType,
+                                         unsigned preferredSessionId,
+                                         OpalMediaType::AutoStartMode autoStart)
+  : OpalMediaTypeDefinition(mediaType, sdpType, preferredSessionId, autoStart)
 {
 }
 
@@ -189,10 +198,16 @@ PString OpalRTPAVPMediaType::GetRTPEncoding() const
   return "rtp/avp";
 }
 
+OpalMediaSession * OpalRTPAVPMediaType::CreateMediaSession(OpalConnection & conn, unsigned sessionID) const
+{
+  return new OpalRTPMediaSession(conn, m_mediaType, sessionID);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalAudioMediaType::OpalAudioMediaType()
-  : OpalRTPAVPMediaType("audio", "audio", 1)
+  : OpalRTPAVPMediaType("audio", "audio", 1, OpalMediaType::ReceiveTransmit)
 {
 }
 
@@ -208,4 +223,3 @@ OpalVideoMediaType::OpalVideoMediaType()
 #endif // OPAL_VIDEO
 
 ///////////////////////////////////////////////////////////////////////////////
-
