@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22204 $
- * $Author: rjongbloed $
- * $Date: 2009-03-18 00:28:06 +0000 (Wed, 18 Mar 2009) $
+ * $Revision: 22553 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
  */
 
 #include <ptlib.h>
@@ -411,7 +411,8 @@ static void SetIncomingCall(OpalMessageBuffer & message, const OpalConnection &
   message->m_param.m_incomingCall.m_product.m_t35Extension     = info.t35Extension;
   message->m_param.m_incomingCall.m_product.m_manufacturerCode = info.manufacturerCode;
 
-  SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_alertingType, network->GetAlertingType());
+  SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_alertingType,   network->GetAlertingType());
+  SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_protocolCallId, connection.GetIdentifier());
 
   PTRACE(4, "OpalC API\tOpalIndIncomingCall: token=\""  << message->m_param.m_incomingCall.m_callToken << "\"\n"
             "  Local  - URL=\"" << message->m_param.m_incomingCall.m_localAddress << "\"\n"
@@ -420,7 +421,8 @@ static void SetIncomingCall(OpalMessageBuffer & message, const OpalConnection &
                   " Display=\"" << message->m_param.m_incomingCall.m_remoteDisplayName << "\"\n"
             "  Dest.  - URL=\"" << message->m_param.m_incomingCall.m_calledAddress << "\""
                     " E.164=\"" << message->m_param.m_incomingCall.m_calledPartyNumber << "\"\n"
-            "  AlertingType=\"" << message->m_param.m_incomingCall.m_alertingType << '"');
+            "  AlertingType=\"" << message->m_param.m_incomingCall.m_alertingType << "\"\n"
+            "        CallID=\"" << message->m_param.m_incomingCall.m_protocolCallId << '"');
 }
 
 
@@ -665,12 +667,12 @@ void SIPEndPoint_C::OnDialogInfoReceived(const SIPDialogNotification & info)
   message->m_param.m_lineAppearance.m_appearance = info.m_local.m_appearance;
 
   if (info.m_initiator) {
-    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_callId, info.m_callId+";to-tag="+info.m_local.m_dialogTag+";from-tag="+info.m_remote.m_dialogTag);
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_callId, info.m_callId+";to-tag="+info.m_remote.m_dialogTag+";from-tag="+info.m_local.m_dialogTag);
     SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyA, GetParticipantName(info.m_local));
     SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyB, GetParticipantName(info.m_remote));
   }
   else {
-    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_callId, info.m_callId+";to-tag="+info.m_remote.m_dialogTag+";from-tag="+info.m_local.m_dialogTag);
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_callId, info.m_callId+";to-tag="+info.m_local.m_dialogTag+";from-tag="+info.m_remote.m_dialogTag);
     SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyA, GetParticipantName(info.m_remote));
     SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyB, GetParticipantName(info.m_local));
   }
@@ -1343,13 +1345,19 @@ void OpalManager_C::HandleSetUpCall(const OpalMessage & command, OpalMessageBuff
 
   OpalConnection::StringOptions options;
   if (!IsNullString(command.m_param.m_callSetUp.m_alertingType))
-    options.SetAt("Alerting-Type", command.m_param.m_callSetUp.m_alertingType);
+    options.SetAt(OPAL_OPT_ALERTING_TYPE, command.m_param.m_callSetUp.m_alertingType);
 
   PString token;
   if (SetUpCall(partyA, command.m_param.m_callSetUp.m_partyB, token, NULL, 0, &options)) {
     SET_MESSAGE_STRING(response, m_param.m_callSetUp.m_partyA, partyA);
     SET_MESSAGE_STRING(response, m_param.m_callSetUp.m_partyB, command.m_param.m_callSetUp.m_partyB);
     SET_MESSAGE_STRING(response, m_param.m_callSetUp.m_callToken, token);
+    PSafePtr<OpalCall> call = FindCallWithLock(token);
+    if (call != NULL) {
+      PSafePtr<OpalConnection> other = call->GetConnection(1);
+      if (other != NULL)
+        SET_MESSAGE_STRING(response, m_param.m_callSetUp.m_protocolCallId, other->GetIdentifier());
+    }
   }
   else
     response.SetError("Call set up failed.");
