@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22553 $
- * $Author: csoutheren $
- * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
+ * $Revision: 23148 $
+ * $Author: rjongbloed $
+ * $Date: 2009-07-28 12:42:40 +0000 (Tue, 28 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -170,6 +170,12 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
             "Destination format:\n" << setw(-1) << destinationFormat);
 
   if (sourceFormat == destinationFormat) {
+    PINDEX framesPerPacket = destinationFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(),
+                                  sourceFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1));
+    PINDEX packetSize = sourceFormat.GetFrameSize()*framesPerPacket;
+    PINDEX packetTime = sourceFormat.GetFrameTime()*framesPerPacket;
+    source.SetDataSize(packetSize, packetTime);
+    sinkStream->SetDataSize(packetSize, packetTime);
     PTRACE(3, "Patch\tAdded direct media stream sink " << *sinkStream);
     return true;
   }
@@ -179,7 +185,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
   if (sink->primaryCodec != NULL) {
     PTRACE(4, "Patch\tCreated primary codec " << sourceFormat << "->" << destinationFormat << " with ID " << id);
 
-    if (!sinkStream->SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(false))) {
+    if (!sinkStream->SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(false), sourceFormat.GetFrameTime())) {
       PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
               << sink->primaryCodec->GetOptimalDataFrameSize(PFalse));
       return false;
@@ -202,7 +208,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 
     PTRACE(4, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
 
-    if (!sinkStream->SetDataSize(sink->secondaryCodec->GetOptimalDataFrameSize(PFalse))) {
+    if (!sinkStream->SetDataSize(sink->secondaryCodec->GetOptimalDataFrameSize(false), sourceFormat.GetFrameTime())) {
       PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
               << sink->secondaryCodec->GetOptimalDataFrameSize(PFalse));
       return false;
@@ -214,7 +220,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
            << " and " << *sink->secondaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
 
-  source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true));
+  source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true), destinationFormat.GetFrameTime());
   return true;
 }
 
@@ -458,12 +464,24 @@ void OpalMediaPatch::Main()
   PTRACE(4, "Patch\tThread started for " << *this);
 	
   bool asynchronous = OnPatchStart();
+  PTimeInterval lastTick;
+
+  /* Note the RTP frame is outside loop so that a) it is more efficient
+     for memory usage, the buffer is only ever increased and not allocated
+     on the heap ever time, and b) the timestamp value embedded into the
+     sourceFrame is needed for correct operation of the jitter buffer and
+     silence frames. It is adjusted by DispatchFrame (really Sink::WriteFrame)
+     each time and passed back in to source.Read() (and eventually the JB) so
+     it knows where it is up to in extracting data from the JB. */
+  RTP_DataFrame sourceFrame(0);
 
   while (source.IsOpen()) {
-
-    RTP_DataFrame sourceFrame(source.GetDataSize());
     sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
-    sourceFrame.SetPayloadSize(0); 
+
+    // We do the following to make sure that the buffer size is large enough,
+    // in case something in previous loop adjusted it
+    sourceFrame.SetPayloadSize(source.GetDataSize());
+    sourceFrame.SetPayloadSize(0);
 
     if (!source.ReadPacket(sourceFrame)) {
       PTRACE(4, "Patch\tThread ended because source read failed");
@@ -479,11 +497,18 @@ void OpalMediaPatch::Main()
       break;
     }
  
-    // Don't starve the CPU if we have idle frames and the no source or destination is synchronous.
-    // Note that performing a Yield is not good enough, as the media patch threads are
-    // high priority and will consume all available CPU if allowed.
-    if (asynchronous)
-      PThread::Sleep(5);
+    /* Don't starve the CPU if we have idle frames and the no source or
+       destination is synchronous. Note that performing a Yield is not good
+       enough, as the media patch threads are high priority and will consume
+       all available CPU if allowed. Also just doing a sleep each time around
+       the loop slows down video where you get clusters of packets thrown at
+       us, want to clear them as quickly as possible out of the UDP OS buffers
+       or we overflow and lose some. Best compromise is to every 100ms, sleep
+       for 10ms so can not use more than about 90% of CPU. */
+    if (asynchronous && PTimer::Tick() - lastTick > 100) {
+      PThread::Sleep(10);
+      lastTick = PTimer::Tick();
+    }
   }
 
   source.OnPatchStop();
