@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22842 $
+ * $Revision: 23260 $
  * $Author: rjongbloed $
- * $Date: 2009-06-11 06:11:12 +0000 (Thu, 11 Jun 2009) $
+ * $Date: 2009-08-24 09:56:39 +0000 (Mon, 24 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -140,7 +140,12 @@ PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFram
   }
 
   // set the output timestamp and marker bit
-  output.front().SetTimestamp(input.GetTimestamp()*outputMediaFormat.GetClockRate()/inputMediaFormat.GetClockRate());
+  unsigned timestamp = input.GetTimestamp();
+  unsigned inClockRate = inputMediaFormat.GetClockRate();
+  unsigned outClockRate = outputMediaFormat.GetClockRate();
+  if (inClockRate != outClockRate)
+    timestamp = (unsigned)((PUInt64)timestamp*outClockRate/inClockRate);
+  output.front().SetTimestamp(timestamp);
   output.front().SetMarker(input.GetMarker());
 
   // set the output payload type directly from the output media format
