@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 22197 $
+ * $Revision: 23294 $
  * $Author: rjongbloed $
- * $Date: 2009-03-12 01:11:14 +0000 (Thu, 12 Mar 2009) $
+ * $Date: 2009-08-28 06:26:26 +0000 (Fri, 28 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -220,7 +220,7 @@ void OpalTransportAddress::SetInternalTransport(WORD port, const char * proto)
   }
 
   // use factory to create transport types
-  transport = PFactory<OpalInternalTransport>::CreateInstance(Left(dollar+1));
+  transport = PFactory<OpalInternalTransport>::CreateInstance(Left(dollar+1).ToLower());
   if (transport == NULL)
     return;
 
@@ -1446,7 +1446,7 @@ static PBoolean SetSSLCertificate(PSSLContext & sslContext,
            << "/CN=" << PProcess::Current().GetName() << '@' << PIPSocket::GetHostName();
     }
     if (!certificate.CreateRoot(name, key)) {
-      PTRACE(0, "MTGW\tCould not create certificate");
+      PTRACE(1, "MTGW\tCould not create certificate");
       return PFalse;
     }
     certificate.Save(certificateFile);
