@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22352 $
+ * $Revision: 22760 $
  * $Author: rjongbloed $
- * $Date: 2009-03-31 07:14:52 +0000 (Tue, 31 Mar 2009) $
+ * $Date: 2009-05-29 02:14:26 +0000 (Fri, 29 May 2009) $
  */
 
 #include <ptlib.h>
@@ -201,12 +201,13 @@ void OpalJitterBuffer::Resume()
 {
   PWaitAndSignal m(bufferMutex);
   if (jitterThread != NULL) {
-    if (shuttingDown) {
-      jitterThread->WaitForTermination();
-      delete jitterThread;
-      jitterThread= NULL;
-    }
+    if (!shuttingDown)
+      return;
+
+    jitterThread->WaitForTermination();
+    delete jitterThread;
   }
+
   shuttingDown = false;
   jitterThread = PThread::Create(PCREATE_NOTIFIER(JitterThreadMain), "RTP Jitter");
   jitterThread->Resume();
