@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22186 $
+ * $Revision: 23381 $
  * $Author: rjongbloed $
- * $Date: 2009-03-10 04:27:59 +0000 (Tue, 10 Mar 2009) $
+ * $Date: 2009-09-08 09:15:25 +0000 (Tue, 08 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -114,12 +114,21 @@ SIPHandler::SIPHandler(SIPEndPoint & ep,
     if (remote.IsEmpty())
       m_remoteAddress = m_addressOfRecord;
     else if (remote.Find('@') != P_MAX_INDEX)
-      m_remoteAddress = remote;
-    else if (m_addressOfRecord.GetHostAddress().IsEquivalent(remote))
-      m_remoteAddress = m_addressOfRecord;
+      m_remoteAddress = remote; // For third party registrations
     else {
-      m_remoteAddress = m_proxy = remote;
-      m_remoteAddress.SetUserName(m_addressOfRecord.GetUserName());
+      SIPURL remoteURL = remote;
+      if (m_addressOfRecord.GetHostAddress().IsEquivalent(remoteURL.GetHostAddress()))
+        m_remoteAddress = m_addressOfRecord;
+      else {
+        /* Note this sets the proxy field because the user has givena full AOR
+           with a domain for "target" and then specified a specific host name
+           which as far as we are concered is the host to talk to. Setting the
+           proxy will prevent SRV lookups or other things that might stop uis
+           from going to that very specific host.
+         */
+        m_remoteAddress = m_proxy = remoteURL;
+        m_remoteAddress.SetUserName(m_addressOfRecord.GetUserName());
+      }
     }
   }
 
@@ -132,6 +141,8 @@ SIPHandler::SIPHandler(SIPEndPoint & ep,
 
 SIPHandler::~SIPHandler() 
 {
+  expireTimer.Stop();
+
   if (m_transport) {
     m_transport->CloseWait();
     delete m_transport;
@@ -197,8 +208,16 @@ OpalTransport * SIPHandler::GetTransport()
   if (m_proxy.IsEmpty())
     m_proxy = endpoint.GetProxy();
 
+  SIPURL url;
+  if (!m_proxy.IsEmpty())
+    url = m_proxy;
+  else {
+    url = GetAddressOfRecord();
+    url.AdjustToDNS();
+  }
+
   // Must specify a network interface or get infinite recursion
-  m_transport = endpoint.CreateTransport(m_proxy.IsEmpty() ? GetAddressOfRecord() : m_proxy, "*");
+  m_transport = endpoint.CreateTransport(url, "*");
   return m_transport;
 }
 
@@ -219,7 +238,7 @@ void SIPHandler::SetExpire(int e)
   // if the expire time is more than 20 mins, retry 10mins before expiry
   // if the expire time is less than 20 mins, retry after half of the expiry time
   if (expire > 0 && state < Unsubscribing)
-    expireTimer.SetInterval(0, (unsigned)(expire < 1200*1000 ? expire/2 : expire-600*1000));
+    expireTimer.SetInterval(0, (unsigned)(expire < 20*60 ? expire/2 : expire-10*60));
 }
 
 
@@ -249,10 +268,30 @@ bool SIPHandler::WriteSIPHandler(OpalTransport & transport)
 }
 
 
+bool SIPHandler::ActivateState(SIPHandler::State newState, unsigned msecs)
+{
+  PTimeInterval startTick = PTimer::Tick();
+  for (;;) {
+    {
+      PSafeLockReadWrite mutex(*this);
+      if (!mutex.IsLocked())
+        return false;
+
+      if (SendRequest(newState))
+        return true;
+    }
+
+    if ((PTimer::Tick() - startTick) > msecs)
+      return false;
+
+    PThread::Sleep(100);
+  }
+}
+
+
 PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
 {
   expireTimer.Stop(false); // Stop automatic retry
-  bool retryLater = false;
 
   if (expire == 0)
     newState = Unsubscribing;
@@ -260,14 +299,26 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
   switch (newState) {
     case Unsubscribing:
       switch (state) {
-        case Unsubscribed :
-          return true;
-
         case Subscribed :
         case Unavailable :
-          break;
+          break;  // Can try and do Unsubscribe
 
-        default : // Are in the process of doing something
+        case Subscribing :
+        case Refreshing :
+        case Restoring :
+          PTRACE(2, "SIP\tCan't send " << newState << " request for " << GetMethod()
+                 << " handler while in " << state << " state, target="
+                 << GetAddressOfRecord() << ", id=" << GetCallID());
+          return false; // Are in the process of doing something
+
+        case Unsubscribed :
+        case Unsubscribing :
+          PTRACE(3, "SIP\tAlready doing " << state << " request for " << GetMethod()
+                 << " handler, target=" << GetAddressOfRecord() << ", id=" << GetCallID());
+          return true;  // Already done or doing it
+
+        default :
+          PAssertAlways(PInvalidParameter);
           return false;
       }
       break;
@@ -275,8 +326,29 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
     case Subscribing :
     case Refreshing :
     case Restoring :
-      if (GetTransport() == NULL) 
-        retryLater = true;
+      switch (state) {
+        case Subscribed :
+        case Unavailable :
+          break;  // Can do subscribe/refresh/restore
+
+        case Refreshing :
+        case Restoring :
+          PTRACE(3, "SIP\tAlready doing " << state << " request for " << GetMethod()
+                 << " handler, target=" << GetAddressOfRecord() << ", id=" << GetCallID());
+          return true; // Already doing it
+
+        case Subscribing :
+        case Unsubscribing :
+        case Unsubscribed :
+          PTRACE(2, "SIP\tCan't send " << newState << " request for " << GetMethod()
+                 << " handler while in " << state << " state, target="
+                 << GetAddressOfRecord() << ", id=" << GetCallID());
+          return false; // Can't restart as are on the way out
+
+        default : // Are in the process of doing something
+          PAssertAlways(PInvalidParameter);
+          return false;
+      }
       break;
 
     default :
@@ -286,7 +358,7 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
 
   SetState(newState);
 
-  if (!retryLater) {
+  if (GetTransport() != NULL) {
     // Restoring or first time, try every interface
     if (newState == Restoring || m_transport->GetInterface().IsEmpty()) {
       PWaitAndSignal mutex(m_transport->GetWriteMutex());
@@ -298,19 +370,21 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
       if (WriteSIPHandler(*m_transport))
         return true;
     }
+
     OnFailed(SIP_PDU::Local_TransportError);
-    retryLater = true;
   }
 
-  if (retryLater) {
-    PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << offlineExpire << " seconds.");
-    OnFailed(SIP_PDU::Local_BadTransportAddress);
-    expireTimer.SetInterval(0, offlineExpire); // Keep trying to get it back
-    SetState(Unavailable);
+  if (newState == Unsubscribing) {
+    // Transport level error, probably never going to get the unsubscribe through
+    SetState(Unsubscribed);
     return true;
   }
 
-  return false;
+  OnFailed(SIP_PDU::Local_BadTransportAddress);
+
+  PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << offlineExpire << " seconds.");
+  expireTimer.SetInterval(0, offlineExpire); // Keep trying to get it back
+  return true;
 }
 
 
@@ -320,6 +394,78 @@ PBoolean SIPHandler::OnReceivedNOTIFY(SIP_PDU & /*response*/)
 }
 
 
+void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response)
+{
+  // Received a response, so collapse the forking on multiple interfaces.
+
+  transactions.Remove(&transaction); // Take this transaction out of list
+
+  // And kill all the rest
+  PSafePtr<SIPTransaction> transToGo;
+  while ((transToGo = transactions.GetAt(0)) != NULL) {
+    transactions.Remove(transToGo);
+    transToGo->Abort();
+  }
+
+  // Finally end connect mode on the transport
+  m_transport->SetInterface(transaction.GetInterface());
+
+  switch (response.GetStatusCode()) {
+    case SIP_PDU::Failure_UnAuthorised :
+    case SIP_PDU::Failure_ProxyAuthenticationRequired :
+      OnReceivedAuthenticationRequired(transaction, response);
+      break;
+
+    case SIP_PDU::Failure_IntervalTooBrief :
+      OnReceivedIntervalTooBrief(transaction, response);
+      break;
+
+    case SIP_PDU::Failure_TemporarilyUnavailable:
+      OnReceivedTemporarilyUnavailable(transaction, response);
+      break;
+
+    case SIP_PDU::Failure_RequestTimeout :
+      OnTransactionFailed(transaction);
+      break;
+
+    default :
+      switch (response.GetStatusCode()/100) {
+        case 1 :
+          // Do nothing on 1xx
+          break;
+
+        case 2 :
+          OnReceivedOK(transaction, response);
+          break;
+
+        default :
+          OnFailed(response.GetStatusCode());
+      }
+  }
+}
+
+
+void SIPHandler::OnReceivedIntervalTooBrief(SIPTransaction & /*transaction*/, SIP_PDU & response)
+{
+  SetExpire(response.GetMIME().GetMinExpires());
+
+  // Restart the transaction with new authentication handler
+  State oldState = state;
+  state = Unavailable;
+  SendRequest(oldState);
+}
+
+
+void SIPHandler::OnReceivedTemporarilyUnavailable(SIPTransaction & /*transaction*/, SIP_PDU & response)
+{
+  OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
+
+  unsigned retryAfter = response.GetMIME().GetInteger("Retry-After", offlineExpire);
+  PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << retryAfter << " seconds.");
+  expireTimer.SetInterval(0, retryAfter); // Have another go in a little bit
+}
+
+
 void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
 {
   bool isProxy = response.GetStatusCode() == SIP_PDU::Failure_ProxyAuthenticationRequired;
@@ -365,6 +511,16 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
       username = proxy.GetUserName();
       password = proxy.GetPassword();
     }
+    else {
+      delete newAuth;
+      PTRACE(1, "SIP\tAuthentication not possible yet.");
+      OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
+      if (expire > 0 && !transaction.IsCanceled()) {
+        PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << offlineExpire << " seconds.");
+        expireTimer.SetInterval(0, offlineExpire); // Keep trying to get it back
+      }
+      return;
+    }
   }
 
   newAuth->SetUsername(username);
@@ -385,15 +541,14 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
   m_username = username;
   m_password = password;
 
-  // And end connect mode on the transport
-  CollapseFork(transaction);
-
   // Restart the transaction with new authentication handler
-  SendRequest(GetState());
+  State oldState = state;
+  state = Unavailable;
+  SendRequest(oldState);
 }
 
 
-void SIPHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
+void SIPHandler::OnReceivedOK(SIPTransaction & /*transaction*/, SIP_PDU & response)
 {
   response.GetMIME().GetProductInfo(m_productInfo);
 
@@ -417,25 +572,6 @@ void SIPHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 
   // reset the number of unsuccesful authentication attempts
   authenticationAttempts = 0;
-
-  CollapseFork(transaction);
-}
-
-
-void SIPHandler::CollapseFork(SIPTransaction & transaction)
-{
-  // Take this transaction out
-  transactions.Remove(&transaction);
-
-  // And kill all the rest
-  PSafePtr<SIPTransaction> transToGo;
-  while ((transToGo = transactions.GetAt(0)) != NULL) {
-    transactions.Remove(transToGo);
-    transToGo->Abort();
-  }
-
-  // Finally end connect mode on the transport
-  m_transport->SetInterface(transaction.GetInterface());
 }
 
 
@@ -460,8 +596,10 @@ void SIPHandler::OnFailed(SIP_PDU::StatusCodes code)
       break;
 
     case SIP_PDU::Local_TransportError :
+    case SIP_PDU::Local_Timeout :
     case SIP_PDU::Failure_RequestTimeout :
     case SIP_PDU::Local_BadTransportAddress :
+    case SIP_PDU::Failure_TemporarilyUnavailable:
       SetState(Unavailable);
       break;
 
@@ -472,7 +610,6 @@ void SIPHandler::OnFailed(SIP_PDU::StatusCodes code)
       SetState(Unsubscribed);
       ShutDown();
   }
-
 }
 
 
@@ -548,18 +685,32 @@ SIPTransaction * SIPRegisterHandler::CreateTransaction(OpalTransport & trans)
     params.m_expire = expire;
 
     if (params.m_contactAddress.IsEmpty()) {
-      // Nothing explicit, put in all the bound interfaces.
-      unsigned qvalue = 1000;
       PString userName = SIPURL(params.m_addressOfRecord).GetUserName();
       OpalTransportAddressArray interfaces = endpoint.GetInterfaceAddresses(true, &trans);
-      for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
-        if (!params.m_contactAddress.IsEmpty())
-          params.m_contactAddress += ", ";
-        SIPURL contact(userName, interfaces[i]);
+      if (params.m_compatibility == SIPRegister::e_CannotRegisterMultipleContacts) {
+        // If translated by STUN then only the external address of the interface is used.
+        SIPURL contact(userName, interfaces[0]);
         contact.Sanitise(SIPURL::ContactURI);
         params.m_contactAddress += contact.AsQuotedString();
-        params.m_contactAddress.sprintf(qvalue < 1000 ? ";q=0.%03u" : ";q=1", qvalue);
-        qvalue -= 1000/interfaces.GetSize();
+      }
+      else {
+        OpalTransportAddress localAddress = trans.GetLocalAddress();
+        unsigned qvalue = 1000;
+        for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
+          /* If fully compliant, put into the contact field all the bound
+             interfaces. If special then we only put into the contact
+             listeners that are on the same interface. If translated by STUN
+             then only the external address of the interface is used. */
+          if (params.m_compatibility != SIPRegister::e_CannotRegisterPrivateContacts || localAddress.IsEquivalent(interfaces[i], true)) {
+            if (!params.m_contactAddress.IsEmpty())
+              params.m_contactAddress += ", ";
+            SIPURL contact(userName, interfaces[i]);
+            contact.Sanitise(SIPURL::ContactURI);
+            params.m_contactAddress += contact.AsQuotedString();
+            params.m_contactAddress.sprintf(qvalue < 1000 ? ";q=0.%03u" : ";q=1", qvalue);
+            qvalue -= 1000/interfaces.GetSize();
+          }
+        }
       }
     }
     else {
@@ -623,6 +774,7 @@ void SIPRegisterHandler::SendStatus(SIP_PDU::StatusCodes code, State state)
   status.m_addressofRecord = GetAddressOfRecord().AsString();
   status.m_productInfo = m_productInfo;
   status.m_reason = code;
+  status.m_userData = m_parameters.m_userData;
 
   switch (state) {
     case Subscribing :
@@ -691,6 +843,10 @@ SIPSubscribeHandler::SIPSubscribeHandler(SIPEndPoint & endpoint, const SIPSubscr
   m_username = params.m_authID;
   m_password = params.m_password;
   m_realm    = params.m_realm;
+
+  // having an agent is the same as having a proxy....near enough
+  if (!params.m_agentAddress.IsEmpty())
+    m_proxy = params.m_agentAddress;
 }
 
 
@@ -706,8 +862,12 @@ SIPTransaction * SIPSubscribeHandler::CreateTransaction(OpalTransport &trans)
   // Default routeSet if there is a proxy
   m_dialog.UpdateRouteSet(m_proxy);
 
-  if (!m_dialog.IsEstablished())
-    m_dialog.SetLocalURI(endpoint.GetRegisteredPartyName(GetAddressOfRecord(), *m_transport));
+  if (!m_dialog.IsEstablished()) {
+    if (!m_parameters.m_from.IsEmpty())
+      m_dialog.SetLocalURI(m_parameters.m_from);
+    else
+      m_dialog.SetLocalURI(endpoint.GetRegisteredPartyName(GetAddressOfRecord(), *m_transport));
+  }
 
   m_parameters.m_expire = state != Unsubscribing ? expire : 0;
   return new SIPSubscribe(endpoint, trans, m_dialog, m_parameters);
@@ -848,7 +1008,7 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
 
 class SIPMwiEventPackageHandler : public SIPEventPackageHandler
 {
-  virtual PString GetContentType() const
+  virtual PCaselessString GetContentType() const
   {
     return "application/simple-message-summary";
   }
@@ -902,7 +1062,7 @@ static SIPEventPackageFactory::Worker<SIPMwiEventPackageHandler> mwiEventPackage
 
 class SIPPresenceEventPackageHandler : public SIPEventPackageHandler
 {
-  virtual PString GetContentType() const
+  virtual PCaselessString GetContentType() const
   {
     return "application/pidf+xml";
   }
@@ -912,8 +1072,12 @@ class SIPPresenceEventPackageHandler : public SIPEventPackageHandler
     SIPURL from = request.GetMIME().GetFrom();
     from.Sanitise(SIPURL::ExternalURI);
 
+    SIPURL to = request.GetMIME().GetTo();
+    to.Sanitise(SIPURL::ExternalURI);
+
     SIPPresenceInfo info;
     info.m_address = from.AsQuotedString();
+    info.m_to      = to.AsQuotedString();
 
     // Check for empty body, if so then is OK, just a ping ...
     if (request.GetEntityBody().IsEmpty()) {
@@ -1013,7 +1177,7 @@ public:
   {
   }
 
-  virtual PString GetContentType() const
+  virtual PCaselessString GetContentType() const
   {
     return "application/dialog-info+xml";
   }
@@ -1081,7 +1245,7 @@ public:
          << m_dialogNotifyVersion++ << "\" state=\"partial\" entity=\""
          << handler.GetAddressOfRecord() << "\">\r\n";
 
-    map<PString, SIPDialogNotification>::iterator iter;
+    std::map<PString, SIPDialogNotification>::iterator iter;
 
     const SIPDialogNotification * info = dynamic_cast<const SIPDialogNotification *>(data);
     if (info != NULL) {
@@ -1104,7 +1268,7 @@ public:
   }
 
   unsigned m_dialogNotifyVersion;
-  map<PString, SIPDialogNotification> m_activeDialogs;
+  std::map<PString, SIPDialogNotification> m_activeDialogs;
 };
 
 static SIPEventPackageFactory::Worker<SIPDialogEventPackageHandler> dialogEventPackageHandler(SIPSubscribe::Dialog);
@@ -1275,17 +1439,22 @@ PBoolean SIPNotifyHandler::SendRequest(SIPHandler::State state)
 
 bool SIPNotifyHandler::SendNotify(const PObject * body)
 {
+  if (!LockReadWrite())
+    return false;
+
   if (m_packageHandler != NULL)
     SetBody(m_packageHandler->OnSendNOTIFY(*this, body));
   else if (body == NULL)
     SetBody(PString::Empty());
   else {
     PStringStream str;
-    str << body;
+    str << *body;
     SetBody(str);
   }
 
-  return SendRequest(Subscribing);
+  UnlockReadWrite();
+
+  return ActivateState(Subscribing, endpoint.GetNonInviteTimeout().GetInterval());
 }
 
 
@@ -1322,7 +1491,7 @@ SIPPublishHandler::~SIPPublishHandler()
 
 SIPTransaction * SIPPublishHandler::CreateTransaction(OpalTransport & t)
 {
-  SetExpire(originalExpire);
+  m_parameters.m_expire = expire;
   return new SIPPublish(endpoint,
                         t, 
                         GetCallID(),
@@ -1457,10 +1626,20 @@ SIPTransaction * SIPPingHandler::CreateTransaction(OpalTransport &t)
 
 //////////////////////////////////////////////////////////////////
 
+/* All of the bwlow search loops run through the list with only
+   PSafeReference rather than PSafeReadOnly, even though they are
+   reading fields from the handler instances. We can get away with
+   this becuase the information being tested, e.g. AOR, is constant
+   for the life of the handler instance, once constructed.
+
+   We need to use PSafeReference as there are some cases where
+   deadlocks can occur when locked handlers look for information
+   from other handlers.
+ */
 unsigned SIPHandlersList::GetCount(SIP_PDU::Methods meth, const PString & eventPackage) const
 {
   unsigned count = 0;
-  for (PSafePtr<SIPHandler> handler(*this, PSafeReference); handler != NULL; ++handler)
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler)
     if (handler->GetState () == SIPHandler::Subscribed &&
         handler->GetMethod() == meth &&
         (eventPackage.IsEmpty() || handler->GetEventPackage() == eventPackage))
@@ -1472,7 +1651,7 @@ unsigned SIPHandlersList::GetCount(SIP_PDU::Methods meth, const PString & eventP
 PStringList SIPHandlersList::GetAddresses(bool includeOffline, SIP_PDU::Methods meth, const PString & eventPackage) const
 {
   PStringList addresses;
-  for (PSafePtr<SIPHandler> handler(*this, PSafeReference); handler != NULL; ++handler)
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler)
     if ((includeOffline ? handler->GetState () != SIPHandler::Unsubscribed
                         : handler->GetState () == SIPHandler::Subscribed) &&
         handler->GetMethod() == meth &&
@@ -1485,11 +1664,12 @@ PStringList SIPHandlersList::GetAddresses(bool includeOffline, SIP_PDU::Methods
 /**
  * Find the SIPHandler object with the specified callID
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & callID, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & callID, PSafetyMode mode)
 {
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler)
-    if (callID == handler->GetCallID())
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (callID == handler->GetCallID() && handler.SetSafetyMode(mode))
       return handler;
+  }
   return NULL;
 }
 
@@ -1497,18 +1677,44 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & cal
 /**
  * Find the SIPHandler object with the specified authRealm
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode mode)
 {
-  PIPSocket::Address realmAddress;
+  PIPSocket::Address handlerRealmAddress;
+  PIPSocket::Address authRealmAddress(authRealm);
 
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (authRealm == handler->GetRealm() && (userName.IsEmpty() || userName == handler->GetUsername()))
+  // if username is specified, look for exact matches
+  if (!userName.IsEmpty()) {
+
+    // look for a match to exact user name and realm
+    for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+      if ( handler->GetUsername() == userName &&
+          (handler->GetRealm().IsEmpty() || handler->GetRealm() == authRealm) &&
+           handler.SetSafetyMode(mode))
+        return handler;
+    }
+
+    // look for a match to exact username and realm as hostname
+    for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+      if (PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
+          handlerRealmAddress == authRealmAddress &&
+          handler->GetUsername() == userName &&
+          handler.SetSafetyMode(mode))
+        return handler;
+    }
+  }
+
+  // look for a match to exact realm
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (handler->GetRealm() == authRealm && handler.SetSafetyMode(mode))
       return handler;
   }
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (PIPSocket::GetHostAddress(handler->GetRealm(), realmAddress))
-      if (realmAddress == PIPSocket::Address(authRealm) && (userName.IsEmpty() || userName == handler->GetUsername()))
-        return handler;
+
+  // look for a match to exact realm as hostname
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
+        handlerRealmAddress == authRealmAddress &&
+        handler.SetSafetyMode(mode))
+      return handler;
   }
   return NULL;
 }
@@ -1521,24 +1727,27 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString &
  * or 6001@seconix.com when registering 6001@seconix.com to
  * sip.seconix.com
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & remoteAddress, SIP_PDU::Methods meth, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & remoteAddress, SIP_PDU::Methods meth, PSafetyMode mode)
 {
   SIPURL remoteURL = remoteAddress;
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (meth == handler->GetMethod() && remoteURL == handler->GetAddressOfRecord())
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (handler->GetMethod() == meth &&
+        handler->GetAddressOfRecord() == remoteURL &&
+        handler.SetSafetyMode(mode))
       return handler;
   }
   return NULL;
 }
 
 
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & aor, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & aor, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode mode)
 {
   SIPURL aorURL = aor;
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (meth == handler->GetMethod() &&
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (handler->GetMethod() == meth &&
         handler->GetAddressOfRecord() == aorURL &&
-        handler->GetEventPackage() == eventPackage)
+        handler->GetEventPackage() == eventPackage &&
+        handler.SetSafetyMode(mode))
       return handler;
   }
   return NULL;
@@ -1550,14 +1759,14 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & aor, S
  * For example, in the above case, the name parameter
  * could be "sip.seconix.com" or "seconix.com".
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByDomain(const PString & name, SIP_PDU::Methods meth, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByDomain(const PString & name, SIP_PDU::Methods meth, PSafetyMode mode)
 {
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-
-    if ((handler->GetMethod() == meth) &&
-        (handler->GetState() != SIPHandler::Unsubscribed) &&
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if ( handler->GetMethod() == meth &&
+         handler->GetState() != SIPHandler::Unsubscribed &&
         (handler->GetAddressOfRecord().GetHostName() == name ||
-        handler->GetAddressOfRecord().GetHostAddress().IsEquivalent(name)))
+         handler->GetAddressOfRecord().GetHostAddress().IsEquivalent(name)) &&
+         handler.SetSafetyMode(mode))
       return handler;
   }
   return NULL;
