@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22902 $
+ * $Revision: 23358 $
  * $Author: rjongbloed $
- * $Date: 2009-06-16 04:58:09 +0000 (Tue, 16 Jun 2009) $
+ * $Date: 2009-09-07 12:04:39 +0000 (Mon, 07 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -112,8 +112,11 @@ static OpalTransportAddress ParseConnectAddress(const PStringArray & tokens, PIN
         else if (tokens[offset+2] == "0.0.0.0") {
           PTRACE(3, "SDP\tConnection address of 0.0.0.0 specified for HOLD request.");
         }
-        else
-          return OpalTransportAddress(tokens[offset+2], port, "udp");
+        else {
+          OpalTransportAddress address(tokens[offset+2], port, "udp");
+          PTRACE(4, "SDP\tParsed connection address " << address);
+          return address;
+        }
       }
       else
       {
@@ -699,8 +702,6 @@ void SDPMediaDescription::Encode(const OpalTransportAddress & commonAddr, ostrea
   PIPSocket::Address commonIP, transportIP;
   if (transportAddress.GetIpAddress(transportIP) && commonAddr.GetIpAddress(commonIP) && commonIP != transportIP)
     connectString = GetConnectAddressString(transportAddress);
-  if (connectString.IsEmpty())
-    connectString = GetConnectAddressString(transportAddress);
   PrintOn(strm, connectString);
 }
 
@@ -1088,30 +1089,6 @@ SDPSessionDescription::SDPSessionDescription(time_t sessionId, unsigned version,
 
 void SDPSessionDescription::PrintOn(ostream & str) const
 {
-  OpalTransportAddress connectionAddress(defaultConnectAddress);
-
-  // see common connect address is needed
-  {
-    OpalTransportAddress descrAddress;
-    PINDEX matched = 0;
-    PINDEX descrMatched = 0;
-    PINDEX i;
-    for (i = 0; i < mediaDescriptions.GetSize(); i++) {
-      if (i == 0)
-        descrAddress = mediaDescriptions[i].GetTransportAddress();
-      if (mediaDescriptions[i].GetTransportAddress() == connectionAddress)
-        ++matched;
-      if (mediaDescriptions[i].GetTransportAddress() == descrAddress)
-        ++descrMatched;
-    }
-    if (connectionAddress != descrAddress) {
-      if ((descrMatched > matched))
-        connectionAddress = descrAddress;
-      else
-        connectionAddress.MakeEmpty();
-    }
-  }
-
   /* encode mandatory session information, note the order is important according to RFC!
      Must be vosiuepcbzkatrm and within the m it is icbka */
   str << "v=" << protocolVersion << "\r\n"
@@ -1122,8 +1099,8 @@ void SDPSessionDescription::PrintOn(ostream & str) const
       << "\r\n"
          "s=" << sessionName << "\r\n";
 
-  if (!connectionAddress.IsEmpty())
-    str << "c=" << GetConnectAddressString(connectionAddress) << "\r\n";
+  if (!defaultConnectAddress.IsEmpty())
+    str << "c=" << GetConnectAddressString(defaultConnectAddress) << "\r\n";
   
   str << bandwidth
       << "t=" << "0 0" << "\r\n";
@@ -1148,7 +1125,7 @@ void SDPSessionDescription::PrintOn(ostream & str) const
   // encode media session information
   for (PINDEX i = 0; i < mediaDescriptions.GetSize(); i++) {
     if (mediaDescriptions[i].PreEncode())
-      mediaDescriptions[i].Encode(connectionAddress, str);
+      mediaDescriptions[i].Encode(defaultConnectAddress, str);
   }
 }
 
