@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22589 $
+ * $Revision: 23493 $
  * $Author: rjongbloed $
- * $Date: 2009-05-12 10:18:55 +0000 (Tue, 12 May 2009) $
+ * $Date: 2009-09-19 02:00:38 +0000 (Sat, 19 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -285,43 +285,104 @@ void SIPURL::ParseAsAddress(const PString & name, const OpalTransportAddress & a
 
 PBoolean SIPURL::InternalParse(const char * cstr, const char * p_defaultScheme)
 {
-  PString defaultScheme = p_defaultScheme != NULL ? p_defaultScheme : "sip";
+  /* This will try to parse an SIP URI according to the RFC3261 EBNF
 
-  displayName = PString::Empty();
-  fieldParameters = PString::Empty();
+        Contact        =  ("Contact" / "m" ) HCOLON
+                          ( STAR / (contact-param *(COMMA contact-param)))
+        contact-param  =  (name-addr / addr-spec) *(SEMI contact-params)
 
+        From           =  ( "From" / "f" ) HCOLON from-spec
+        from-spec      =  ( name-addr / addr-spec ) *( SEMI from-param )
+
+        Record-Route   =  "Record-Route" HCOLON rec-route *(COMMA rec-route)
+        rec-route      =  name-addr *( SEMI rr-param )
+
+        Reply-To       =  "Reply-To" HCOLON rplyto-spec
+        rplyto-spec    =  ( name-addr / addr-spec ) *( SEMI rplyto-param )
+
+        Route          =  "Route" HCOLON route-param *(COMMA route-param)
+        route-param    =  name-addr *( SEMI rr-param )
+
+        To             =  ( "To" / "t" ) HCOLON
+                          ( name-addr / addr-spec ) *( SEMI to-param )
+
+
+     For all of the above the field parameters (contact-params, to-param etc)
+     all degenerate into special cases of the generic-param production. The
+     generic-param is always included as an option for all these fields as
+     well, so for general parsing we just need to parse the EBNF:
+
+        generic-param  =  token [ EQUAL gen-value ]
+        gen-value      =  token / host / quoted-string
+
+     The URI is then embedded in:
+
+        name-addr      =  [ display-name ] LAQUOT addr-spec RAQUOT
+        addr-spec      =  SIP-URI / SIPS-URI / absoluteURI
+        display-name   =  *(token LWS) / quoted-string
+
+        token          =  1*(alphanum / "-" / "." / "!" / "%" / "*"
+                             / "_" / "+" / "`" / "'" / "~" )
+
+        quoted-string  =  SWS DQUOTE *(qdtext / quoted-pair ) DQUOTE
+        qdtext         =  LWS / %x21 / %x23-5B / %x5D-7E
+                                / UTF8-NONASCII
+        quoted-pair    =  "\" (%x00-09 / %x0B-0C / %x0E-7F)
+
+
+     So, what all that means is we distinguish between name-addr and addr-spec
+     by the '<' character, but we can't just search for it if there is a
+     quoted-string before it in the optional display-name field. So check for
+     that and remove it first, then look for the the '<' which tells us if
+     there COULD be a generic-param later.
+   */
+
+  displayName.MakeEmpty();
+  fieldParameters.MakeEmpty();
+
+  while (isspace(*cstr))
+    cstr++;
   PString str = cstr;
 
+  PINDEX endQuote = 0;
+  if (str[0] == '"') {
+    do {
+      endQuote = str.Find('"', endQuote+1);
+      if (endQuote == P_MAX_INDEX) {
+        PTRACE(1, "SIP\tNo closing double quote in URI: " << str);
+        return false; // No close quote!
+      }
+    } while (str[endQuote-1] == '\\');
+
+    displayName = str(1, endQuote-1);
+
+    PINDEX backslash;
+    while ((backslash = displayName.Find('\\')) != P_MAX_INDEX)
+      displayName.Delete(backslash, 1);
+  }
+
   // see if URL is just a URI or it contains a display address as well
-  PINDEX startBracket = str.FindLast('<');
+  PINDEX startBracket = str.Find('<', endQuote);
   PINDEX endBracket = str.Find('>', startBracket);
 
+  const char * defaultScheme = p_defaultScheme != NULL ? p_defaultScheme : "sip";
+
   // see if URL is just a URI or it contains a display address as well
   if (startBracket == P_MAX_INDEX || endBracket == P_MAX_INDEX) {
     if (!PURL::InternalParse(cstr, defaultScheme))
       return false;
-    }
+  }
   else {
     // get the URI from between the angle brackets
     if (!PURL::InternalParse(str(startBracket+1, endBracket-1), defaultScheme))
-      return PFalse;
+      return false;
 
     fieldParameters = str.Mid(endBracket+1).Trim();
 
-    // See if display address quoted
-    PINDEX endQuote = str.FindLast('"', startBracket);
-    PINDEX startQuote = str.Find('"');
-    if (startQuote == P_MAX_INDEX || endQuote == P_MAX_INDEX || startQuote >= endQuote) {
-      // There are no double quotes around the display name, so take
-      // everything before the start angle bracket
+    if (endQuote == 0) {
+      // There were no double quotes around the display name, take
+      // everything before the start angle bracket, sans whitespace
       displayName = str.Left(startBracket).Trim();
-      }
-    else {
-      // Trim quotes off
-      displayName = str(startQuote+1, endQuote-1);
-      PINDEX backslash;
-      while ((backslash = displayName.Find('\\')) != P_MAX_INDEX)
-        displayName.Delete(backslash, 1);
     }
     }
 
@@ -584,9 +645,10 @@ PBoolean SIPMIMEInfo::IsContentLengthPresent() const
 }
 
 
-PString SIPMIMEInfo::GetContentType() const
+PCaselessString SIPMIMEInfo::GetContentType(bool includeParameters) const
 {
-  return GetString("Content-Type");
+  PCaselessString str = GetString("Content-Type");
+  return str.Left(includeParameters ? P_MAX_INDEX : str.Find(';')).Trim();
 }
 
 
@@ -596,7 +658,7 @@ void SIPMIMEInfo::SetContentType(const PString & v)
 }
 
 
-PString SIPMIMEInfo::GetContentEncoding() const
+PCaselessString SIPMIMEInfo::GetContentEncoding() const
 {
   return GetString("Content-Encoding");
 }
@@ -1008,7 +1070,7 @@ void SIPMIMEInfo::SetEvent(const PString & v)
 }
 
 
-PString SIPMIMEInfo::GetSubscriptionState() const
+PCaselessString SIPMIMEInfo::GetSubscriptionState() const
 {
   return GetString("Subscription-State");       // no compact form
 }
@@ -1046,7 +1108,7 @@ void SIPMIMEInfo::SetOrganization(const PString & v)
 
 static const char UserAgentTokenChars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.!%*_+`'~";
 
-void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
+void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info) const
 {
   PCaselessString str = GetUserAgent();
   if (str.IsEmpty()) {
@@ -1231,7 +1293,13 @@ static bool LocateFieldParameter(const PString & fieldValue, const PString & par
   PINDEX semicolon = (PINDEX)-1;
   while ((semicolon = fieldValue.Find(';', semicolon+1)) != P_MAX_INDEX) {
     start = fieldValue.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.!%*_+`'~", semicolon+1);
-    if (start != P_MAX_INDEX && fieldValue[start] == '=' && (fieldValue(semicolon+1, start-1) *= paramName)) {
+    if (start > fieldValue.GetLength()) {
+      if (fieldValue(semicolon+1, start) *= paramName) {
+        end = start;
+        return true;
+      }
+    }
+    else if ((fieldValue[start] == '=') && (fieldValue(semicolon+1, start-1) *= paramName)) {
       start++;
       end = fieldValue.FindOneOf("()<>@,;:\\\"/[]?{}= \t", start)-1;
       return true;
@@ -1257,8 +1325,11 @@ PString SIPMIMEInfo::InsertFieldParameter(const PString & fieldValue,
 {
 
   PINDEX start, end;
-  if (LocateFieldParameter(fieldValue, paramName, start, end))
-    return fieldValue.Left(start) + newValue + fieldValue.Mid(end+1);
+  if (LocateFieldParameter(fieldValue, paramName, start, end)) {
+    if (start != end)
+      return fieldValue.Left(start) + newValue + fieldValue.Mid(end+1);
+    return fieldValue.Left(start) + "=" + newValue + fieldValue.Mid(end+1);
+  }
 
   PStringStream newField;
   newField << fieldValue << ';' << paramName << '=' << newValue;
@@ -1827,7 +1898,7 @@ void SIP_PDU::Construct(Methods meth,
             dialog.GetRemoteURI().AsQuotedString(),
             dialog.GetLocalURI().AsQuotedString(),
             dialog.GetCallID(),
-            dialog.GetNextCSeq(),
+            dialog.GetNextCSeq(dialog.IsEstablished() && meth != Method_ACK ? 1 : 0),
             via.GetHostAddress());
 
   SetRoute(dialog.GetRouteSet()); // Possibly adjust the URI and the route
@@ -1971,14 +2042,29 @@ bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response, SIPEnd
       viaAddress = param;
 
     // received is present
+    bool received = false;
     param = SIPMIMEInfo::ExtractFieldParameter(viaList.front(), "received");
-    if (!param.IsEmpty()) 
+    if (!param.IsEmpty()) {
       viaAddress = param;
+      received = true;
+    }
 
-    // rport is present
-    param = SIPMIMEInfo::ExtractFieldParameter(viaList.front(), "rport");
-    if (!param.IsEmpty()) 
-      viaPort = param;
+    // rport is present. be careful to distinguish between not present and empty
+    PIPSocket::Address remoteIp;
+    WORD remotePort;
+    transport.GetLastReceivedAddress().GetIpAndPort(remoteIp, remotePort);
+    {
+      PINDEX start, end;
+      if (LocateFieldParameter(viaList.front(), "rport", start, end)) {
+        param = viaList.front()(start, end);
+        if (!param.IsEmpty()) 
+          viaPort = param;
+        else
+          viaPort = remotePort;
+        if (!received) 
+          viaAddress = remoteIp.AsString();
+      }
+    }
 
     newAddress = OpalTransportAddress(viaAddress+":"+viaPort, defaultPort, (proto *= "TCP") ? "tcp$" : "udp$");
   }
@@ -2294,8 +2380,8 @@ PString SIP_PDU::GetTransactionID() const
 
 SDPSessionDescription * SIP_PDU::GetSDP()
 {
-  if (m_SDP == NULL && (mime.GetContentType() *= "application/sdp")) {
-    m_SDP = new SDPSessionDescription();
+  if (m_SDP == NULL && mime.GetContentType() == "application/sdp") {
+    m_SDP = new SDPSessionDescription(0, 0, OpalTransportAddress());
     if (!m_SDP->Decode(entityBody)) {
       delete m_SDP;
       m_SDP = NULL;
@@ -2306,11 +2392,18 @@ SDPSessionDescription * SIP_PDU::GetSDP()
 }
 
 
+void SIP_PDU::SetSDP(SDPSessionDescription * sdp)
+{
+  delete m_SDP;
+  m_SDP = sdp;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////////
 
 SIPDialogContext::SIPDialogContext()
   : m_callId(SIPTransaction::GenerateCallID())
-  , m_lastSentCSeq(0)
+  , m_lastSentCSeq(1)
   , m_lastReceivedCSeq(0)
 {
 }
@@ -2447,7 +2540,7 @@ SIPTransaction::SIPTransaction(SIPEndPoint & ep,
   , transport(trans)
 {
   Construct(minRetryTime, maxRetryTime);
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " created.");
+  PTRACE(4, "SIP\tTransaction created.");
 }
 
 
@@ -2460,7 +2553,7 @@ SIPTransaction::SIPTransaction(SIPConnection & conn,
 {
   connection = &conn;
   Construct();
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " created.");
+  PTRACE(4, "SIP\t" << meth << " transaction id=" << GetTransactionID() << " created.");
 }
 
 
@@ -2492,23 +2585,24 @@ void SIPTransaction::Construct(Methods method, SIPDialogContext & dialog)
 
 SIPTransaction::~SIPTransaction()
 {
-  PTRACE_IF(1, state < Terminated_Success, "SIP\tDestroying transaction " << mime.GetCSeq() << " which is not yet terminated.");
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " destroyed.");
+  PTRACE_IF(1, state < Terminated_Success, "SIP\tDestroying transaction id="
+            << GetTransactionID() << " which is not yet terminated.");
+  PTRACE(4, "SIP\tTransaction id=" << GetTransactionID() << " destroyed.");
 }
 
 
 PBoolean SIPTransaction::Start()
 {
-  if (state == Completed)
+    if (state == Completed)
     return PTrue;
 
-  if (connection != NULL)
-    connection->OnStartTransaction(*this);
+    if (connection != NULL)
+      connection->OnStartTransaction(*this);
 
-  endpoint.AddTransaction(this);
+    endpoint.AddTransaction(this);
 
-  if (state != NotStarted) {
-    PAssertAlways(PLogicError);
+    if (state != NotStarted) {
+      PAssertAlways(PLogicError);
     return PFalse;
   }
 
@@ -2574,11 +2668,14 @@ PBoolean SIPTransaction::Cancel()
   PSafeLockReadWrite lock(*this);
 
   if (state == NotStarted || state >= Cancelling) {
-    PTRACE(3, "SIP\tTransaction " << mime.GetCSeq() << " cannot be cancelled as in state " << state);
+    PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " cannot be cancelled as in state " << state);
     return PFalse;
   }
 
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " cancelled.");
+  PTRACE(4, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " cancelled.");
+  state = Cancelling;
+  retry = 0;
+  retryTimer = retryTimeoutMin;
   completionTimer = endpoint.GetPduCleanUpTimeout();
   return ResendCANCEL();
 }
@@ -2587,7 +2684,9 @@ PBoolean SIPTransaction::Cancel()
 void SIPTransaction::Abort()
 {
   if (LockReadWrite()) {
-    SetTerminated(Terminated_Aborted);
+    PTRACE(4, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " aborted.");
+    if (!IsCompleted())
+      SetTerminated(Terminated_Aborted);
     UnlockReadWrite();
   }
 }
@@ -2616,16 +2715,7 @@ bool SIPTransaction::ResendCANCEL()
   PStringList viaList = mime.GetViaList();
   cancel.GetMIME().SetVia(viaList.front());
 
-  if (!SendPDU(cancel))
-    return false;
-
-  if (state < Cancelling) {
-    state = Cancelling;
-    retry = 0;
-    retryTimer = retryTimeoutMin;
-  }
-
-  return true;
+  return SendPDU(cancel);
 }
 
 
@@ -2663,7 +2753,7 @@ PBoolean SIPTransaction::OnReceivedResponse(SIP_PDU & response)
      */
   if (IsInProgress()) {
     if (response.GetStatusCode()/100 == 1) {
-      PTRACE(3, "SIP\tTransaction " << cseq << " proceeding.");
+      PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " proceeding.");
 
       if (state == Trying)
         state = Proceeding;
@@ -2680,7 +2770,7 @@ PBoolean SIPTransaction::OnReceivedResponse(SIP_PDU & response)
         completionTimer = endpoint.GetNonInviteTimeout();
     }
     else {
-      PTRACE(3, "SIP\tTransaction " << cseq << " completed.");
+      PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " completed.");
       state = Completed;
       statusCode = response.GetStatusCode();
     }
@@ -2723,15 +2813,6 @@ void SIPTransaction::OnRetry(PTimer &, INT)
     return;
   }
 
-  if (state == Cancelling) {
-    if (!ResendCANCEL())
-      return;
-  }
-  else {
-    if (!SendPDU(*this))
-      return;
-  }
-
   if (state > Trying)
     retryTimer = retryTimeoutMax;
   else {
@@ -2741,7 +2822,13 @@ void SIPTransaction::OnRetry(PTimer &, INT)
     retryTimer = timeout;
   }
 
-  PTRACE(3, "SIP\tTransaction " << mime.GetCSeq() << " timeout, making retry " << retry << ", timeout " << retryTimer);
+  PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID()
+         << " timeout, making retry " << retry << ", timeout " << retryTimer);
+
+  if (state == Cancelling)
+    ResendCANCEL();
+  else
+    SendPDU(*this);
 }
 
 
@@ -2805,7 +2892,7 @@ void SIPTransaction::SetTerminated(States newState)
 
   if (state >= Terminated_Success) {
     PTRACE_IF(3, newState != Terminated_Success, "SIP\tTried to set state " << StateNames[newState] 
-              << " for transaction " << mime.GetCSeq()
+              << " for " << GetMethod() << " transaction id=" << GetTransactionID()
               << " but already terminated ( " << StateNames[state] << ')');
     return;
   }
@@ -2813,14 +2900,15 @@ void SIPTransaction::SetTerminated(States newState)
   States oldState = state;
   
   state = newState;
-  PTRACE(3, "SIP\tSet state " << StateNames[newState] << " for transaction " << mime.GetCSeq());
+  PTRACE(3, "SIP\tSet state " << StateNames[newState] << " for "
+         << GetMethod() << " transaction id=" << GetTransactionID());
 
   // Transaction failed, tell the endpoint
   if (state > Terminated_Success) {
     switch (state) {
       case Terminated_Timeout :
       case Terminated_RetriesExceeded:
-        statusCode = SIP_PDU::Failure_RequestTimeout;
+        statusCode = SIP_PDU::Local_Timeout;
         break;
 
       case Terminated_TransportError :
@@ -2862,12 +2950,6 @@ SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport, cons
   SetAllow(connection.GetEndPoint().GetAllowedMethods());
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
 
-  m_SDP = new SDPSessionDescription();
-  if (!connection.OnSendSDP(false, rtpSessions, *m_SDP)) {
-    delete m_SDP;
-    m_SDP = NULL;
-  }
-
   connection.OnCreatingINVITE(*this);
 }
 
@@ -2914,6 +2996,8 @@ SIPRegister::Params::Params()
   , m_restoreTime(30)
   , m_minRetryTime(PMaxTimeInterval)
   , m_maxRetryTime(PMaxTimeInterval)
+  , m_userData(NULL)
+  , m_compatibility(SIPRegister::e_FullyCompliant)
 {
 }
 
@@ -2988,6 +3072,12 @@ PObject::Comparison SIPSubscribe::EventPackage::InternalCompare(PINDEX offset, P
   if (myIdPtr == NULL && theirIdPtr == NULL)
     return EqualTo;
 
+  if (myIdPtr == NULL)
+    return LessThan;
+
+  if (theirIdPtr == NULL)
+    return GreaterThan;
+
   const char * myIdEnd = strchr(myIdPtr, ';');
   PINDEX myIdLen = myIdEnd != NULL ? myIdEnd - myIdPtr : strlen(myIdPtr);
 
@@ -3010,6 +3100,7 @@ SIPSubscribe::Params::Params(PredefinedPackages pkg)
   , m_restoreTime(30)
   , m_minRetryTime(PMaxTimeInterval)
   , m_maxRetryTime(PMaxTimeInterval)
+  , m_userData(NULL)
 {
 }
 
@@ -3154,14 +3245,12 @@ void SIPRefer::Construct(SIPConnection & connection, OpalTransport & /*transport
 SIPReferNotify::SIPReferNotify(SIPConnection & connection, OpalTransport & transport, StatusCodes code)
   : SIPTransaction(connection, transport, Method_NOTIFY)
 {
-  PStringStream str;
-  
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
-  mime.SetSubscriptionState("terminated;reason=noresource"); // Do not keep an internal state
+  mime.SetSubscriptionState(code < Successful_OK ? "active" : "terminated;reason=noresource");
   mime.SetEvent("refer");
-  mime.SetContentType("message/sipfrag;version=2.0");
-
+  mime.SetContentType("message/sipfrag");
 
+  PStringStream str;
   str << "SIP/" << versionMajor << '.' << versionMinor << " " << code << " " << GetStatusCodeDescription(code);
   entityBody = str;
 }
